# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Type, Iterable, cast

import httpx

from .regions import (
    RegionsResource,
    AsyncRegionsResource,
    RegionsResourceWithRawResponse,
    AsyncRegionsResourceWithRawResponse,
    RegionsResourceWithStreamingResponse,
    AsyncRegionsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from .previews import (
    PreviewsResource,
    AsyncPreviewsResource,
    PreviewsResourceWithRawResponse,
    AsyncPreviewsResourceWithRawResponse,
    PreviewsResourceWithStreamingResponse,
    AsyncPreviewsResourceWithStreamingResponse,
)
from .searches import (
    SearchesResource,
    AsyncSearchesResource,
    SearchesResourceWithRawResponse,
    AsyncSearchesResourceWithRawResponse,
    SearchesResourceWithStreamingResponse,
    AsyncSearchesResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from .pools.pools import (
    PoolsResource,
    AsyncPoolsResource,
    PoolsResourceWithRawResponse,
    AsyncPoolsResourceWithRawResponse,
    PoolsResourceWithStreamingResponse,
    AsyncPoolsResourceWithStreamingResponse,
)
from ...pagination import SyncSinglePage, AsyncSinglePage
from ..._base_client import AsyncPaginator, make_request_options
from .monitors.monitors import (
    MonitorsResource,
    AsyncMonitorsResource,
    MonitorsResourceWithRawResponse,
    AsyncMonitorsResourceWithRawResponse,
    MonitorsResourceWithStreamingResponse,
    AsyncMonitorsResourceWithStreamingResponse,
)
from ...types.load_balancers import (
    SteeringPolicy,
    SessionAffinity,
    load_balancer_edit_params,
    load_balancer_create_params,
    load_balancer_update_params,
)
from .monitor_groups.monitor_groups import (
    MonitorGroupsResource,
    AsyncMonitorGroupsResource,
    MonitorGroupsResourceWithRawResponse,
    AsyncMonitorGroupsResourceWithRawResponse,
    MonitorGroupsResourceWithStreamingResponse,
    AsyncMonitorGroupsResourceWithStreamingResponse,
)
from ...types.load_balancers.rules_param import RulesParam
from ...types.load_balancers.default_pools import DefaultPools
from ...types.load_balancers.load_balancer import LoadBalancer
from ...types.load_balancers.steering_policy import SteeringPolicy
from ...types.load_balancers.session_affinity import SessionAffinity
from ...types.load_balancers.random_steering_param import RandomSteeringParam
from ...types.load_balancers.adaptive_routing_param import AdaptiveRoutingParam
from ...types.load_balancers.location_strategy_param import LocationStrategyParam
from ...types.load_balancers.load_balancer_delete_response import LoadBalancerDeleteResponse
from ...types.load_balancers.session_affinity_attributes_param import SessionAffinityAttributesParam

__all__ = ["LoadBalancersResource", "AsyncLoadBalancersResource"]


class LoadBalancersResource(SyncAPIResource):
    @cached_property
    def monitors(self) -> MonitorsResource:
        return MonitorsResource(self._client)

    @cached_property
    def monitor_groups(self) -> MonitorGroupsResource:
        return MonitorGroupsResource(self._client)

    @cached_property
    def pools(self) -> PoolsResource:
        return PoolsResource(self._client)

    @cached_property
    def previews(self) -> PreviewsResource:
        return PreviewsResource(self._client)

    @cached_property
    def regions(self) -> RegionsResource:
        return RegionsResource(self._client)

    @cached_property
    def searches(self) -> SearchesResource:
        return SearchesResource(self._client)

    @cached_property
    def with_raw_response(self) -> LoadBalancersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return LoadBalancersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LoadBalancersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return LoadBalancersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        zone_id: str,
        default_pools: SequenceNotStr[DefaultPools],
        fallback_pool: str,
        name: str,
        adaptive_routing: AdaptiveRoutingParam | Omit = omit,
        country_pools: Dict[str, SequenceNotStr[str]] | Omit = omit,
        description: str | Omit = omit,
        location_strategy: LocationStrategyParam | Omit = omit,
        networks: SequenceNotStr[str] | Omit = omit,
        pop_pools: Dict[str, SequenceNotStr[str]] | Omit = omit,
        proxied: bool | Omit = omit,
        random_steering: RandomSteeringParam | Omit = omit,
        region_pools: Dict[str, SequenceNotStr[str]] | Omit = omit,
        rules: Iterable[RulesParam] | Omit = omit,
        session_affinity: SessionAffinity | Omit = omit,
        session_affinity_attributes: SessionAffinityAttributesParam | Omit = omit,
        session_affinity_ttl: float | Omit = omit,
        steering_policy: SteeringPolicy | Omit = omit,
        ttl: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LoadBalancer:
        """
        Create a new load balancer.

        Args:
          default_pools: A list of pool IDs ordered by their failover priority. Pools defined here are
              used by default, or when region_pools are not configured for a given region.

          fallback_pool: The pool ID to use when all other pools are detected as unhealthy.

          name: The DNS hostname to associate with your Load Balancer. If this hostname already
              exists as a DNS record in Cloudflare's DNS, the Load Balancer will take
              precedence and the DNS record will not be used.

          adaptive_routing: Controls features that modify the routing of requests to pools and origins in
              response to dynamic conditions, such as during the interval between active
              health monitoring requests. For example, zero-downtime failover occurs
              immediately when an origin becomes unavailable due to HTTP 521, 522, or 523
              response codes. If there is another healthy origin in the same pool, the request
              is retried once against this alternate origin.

          country_pools: A mapping of country codes to a list of pool IDs (ordered by their failover
              priority) for the given country. Any country not explicitly defined will fall
              back to using the corresponding region_pool mapping if it exists else to
              default_pools.

          description: Object description.

          location_strategy: Controls location-based steering for non-proxied requests. See `steering_policy`
              to learn how steering is affected.

          networks: List of networks where Load Balancer or Pool is enabled.

          pop_pools: Enterprise only: A mapping of Cloudflare PoP identifiers to a list of pool IDs
              (ordered by their failover priority) for the PoP (datacenter). Any PoPs not
              explicitly defined will fall back to using the corresponding country_pool, then
              region_pool mapping if it exists else to default_pools.

          proxied: Whether the hostname should be gray clouded (false) or orange clouded (true).

          random_steering: Configures pool weights.

              - `steering_policy="random"`: A random pool is selected with probability
                proportional to pool weights.
              - `steering_policy="least_outstanding_requests"`: Use pool weights to scale each
                pool's outstanding requests.
              - `steering_policy="least_connections"`: Use pool weights to scale each pool's
                open connections.

          region_pools: A mapping of region codes to a list of pool IDs (ordered by their failover
              priority) for the given region. Any regions not explicitly defined will fall
              back to using default_pools.

          rules: BETA Field Not General Access: A list of rules for this load balancer to
              execute.

          session_affinity: Specifies the type of session affinity the load balancer should use unless
              specified as `"none"`. The supported types are: - `"cookie"`: On the first
              request to a proxied load balancer, a cookie is generated, encoding information
              of which origin the request will be forwarded to. Subsequent requests, by the
              same client to the same load balancer, will be sent to the origin server the
              cookie encodes, for the duration of the cookie and as long as the origin server
              remains healthy. If the cookie has expired or the origin server is unhealthy,
              then a new origin server is calculated and used. - `"ip_cookie"`: Behaves the
              same as `"cookie"` except the initial origin selection is stable and based on
              the client's ip address. - `"header"`: On the first request to a proxied load
              balancer, a session key based on the configured HTTP headers (see
              `session_affinity_attributes.headers`) is generated, encoding the request
              headers used for storing in the load balancer session state which origin the
              request will be forwarded to. Subsequent requests to the load balancer with the
              same headers will be sent to the same origin server, for the duration of the
              session and as long as the origin server remains healthy. If the session has
              been idle for the duration of `session_affinity_ttl` seconds or the origin
              server is unhealthy, then a new origin server is calculated and used. See
              `headers` in `session_affinity_attributes` for additional required
              configuration.

          session_affinity_attributes: Configures attributes for session affinity.

          session_affinity_ttl: Time, in seconds, until a client's session expires after being created. Once the
              expiry time has been reached, subsequent requests may get sent to a different
              origin server. The accepted ranges per `session_affinity` policy are: -
              `"cookie"` / `"ip_cookie"`: The current default of 23 hours will be used unless
              explicitly set. The accepted range of values is between [1800, 604800]. -
              `"header"`: The current default of 1800 seconds will be used unless explicitly
              set. The accepted range of values is between [30, 3600]. Note: With session
              affinity by header, sessions only expire after they haven't been used for the
              number of seconds specified.

          steering_policy: Steering Policy for this load balancer.

              - `"off"`: Use `default_pools`.
              - `"geo"`: Use `region_pools`/`country_pools`/`pop_pools`. For non-proxied
                requests, the country for `country_pools` is determined by
                `location_strategy`.
              - `"random"`: Select a pool randomly.
              - `"dynamic_latency"`: Use round trip time to select the closest pool in
                default_pools (requires pool health checks).
              - `"proximity"`: Use the pools' latitude and longitude to select the closest
                pool using the Cloudflare PoP location for proxied requests or the location
                determined by `location_strategy` for non-proxied requests.
              - `"least_outstanding_requests"`: Select a pool by taking into consideration
                `random_steering` weights, as well as each pool's number of outstanding
                requests. Pools with more pending requests are weighted proportionately less
                relative to others.
              - `"least_connections"`: Select a pool by taking into consideration
                `random_steering` weights, as well as each pool's number of open connections.
                Pools with more open connections are weighted proportionately less relative to
                others. Supported for HTTP/1 and HTTP/2 connections.
              - `""`: Will map to `"geo"` if you use
                `region_pools`/`country_pools`/`pop_pools` otherwise `"off"`.

          ttl: Time to live (TTL) of the DNS entry for the IP address returned by this load
              balancer. This only applies to gray-clouded (unproxied) load balancers.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/load_balancers", zone_id=zone_id),
            body=maybe_transform(
                {
                    "default_pools": default_pools,
                    "fallback_pool": fallback_pool,
                    "name": name,
                    "adaptive_routing": adaptive_routing,
                    "country_pools": country_pools,
                    "description": description,
                    "location_strategy": location_strategy,
                    "networks": networks,
                    "pop_pools": pop_pools,
                    "proxied": proxied,
                    "random_steering": random_steering,
                    "region_pools": region_pools,
                    "rules": rules,
                    "session_affinity": session_affinity,
                    "session_affinity_attributes": session_affinity_attributes,
                    "session_affinity_ttl": session_affinity_ttl,
                    "steering_policy": steering_policy,
                    "ttl": ttl,
                },
                load_balancer_create_params.LoadBalancerCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[LoadBalancer]._unwrapper,
            ),
            cast_to=cast(Type[LoadBalancer], ResultWrapper[LoadBalancer]),
        )

    def update(
        self,
        load_balancer_id: str,
        *,
        zone_id: str,
        default_pools: SequenceNotStr[DefaultPools],
        fallback_pool: str,
        name: str,
        adaptive_routing: AdaptiveRoutingParam | Omit = omit,
        country_pools: Dict[str, SequenceNotStr[str]] | Omit = omit,
        description: str | Omit = omit,
        enabled: bool | Omit = omit,
        location_strategy: LocationStrategyParam | Omit = omit,
        networks: SequenceNotStr[str] | Omit = omit,
        pop_pools: Dict[str, SequenceNotStr[str]] | Omit = omit,
        proxied: bool | Omit = omit,
        random_steering: RandomSteeringParam | Omit = omit,
        region_pools: Dict[str, SequenceNotStr[str]] | Omit = omit,
        rules: Iterable[RulesParam] | Omit = omit,
        session_affinity: SessionAffinity | Omit = omit,
        session_affinity_attributes: SessionAffinityAttributesParam | Omit = omit,
        session_affinity_ttl: float | Omit = omit,
        steering_policy: SteeringPolicy | Omit = omit,
        ttl: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LoadBalancer:
        """
        Update a configured load balancer.

        Args:
          default_pools: A list of pool IDs ordered by their failover priority. Pools defined here are
              used by default, or when region_pools are not configured for a given region.

          fallback_pool: The pool ID to use when all other pools are detected as unhealthy.

          name: The DNS hostname to associate with your Load Balancer. If this hostname already
              exists as a DNS record in Cloudflare's DNS, the Load Balancer will take
              precedence and the DNS record will not be used.

          adaptive_routing: Controls features that modify the routing of requests to pools and origins in
              response to dynamic conditions, such as during the interval between active
              health monitoring requests. For example, zero-downtime failover occurs
              immediately when an origin becomes unavailable due to HTTP 521, 522, or 523
              response codes. If there is another healthy origin in the same pool, the request
              is retried once against this alternate origin.

          country_pools: A mapping of country codes to a list of pool IDs (ordered by their failover
              priority) for the given country. Any country not explicitly defined will fall
              back to using the corresponding region_pool mapping if it exists else to
              default_pools.

          description: Object description.

          enabled: Whether to enable (the default) this load balancer.

          location_strategy: Controls location-based steering for non-proxied requests. See `steering_policy`
              to learn how steering is affected.

          networks: List of networks where Load Balancer or Pool is enabled.

          pop_pools: Enterprise only: A mapping of Cloudflare PoP identifiers to a list of pool IDs
              (ordered by their failover priority) for the PoP (datacenter). Any PoPs not
              explicitly defined will fall back to using the corresponding country_pool, then
              region_pool mapping if it exists else to default_pools.

          proxied: Whether the hostname should be gray clouded (false) or orange clouded (true).

          random_steering: Configures pool weights.

              - `steering_policy="random"`: A random pool is selected with probability
                proportional to pool weights.
              - `steering_policy="least_outstanding_requests"`: Use pool weights to scale each
                pool's outstanding requests.
              - `steering_policy="least_connections"`: Use pool weights to scale each pool's
                open connections.

          region_pools: A mapping of region codes to a list of pool IDs (ordered by their failover
              priority) for the given region. Any regions not explicitly defined will fall
              back to using default_pools.

          rules: BETA Field Not General Access: A list of rules for this load balancer to
              execute.

          session_affinity: Specifies the type of session affinity the load balancer should use unless
              specified as `"none"`. The supported types are: - `"cookie"`: On the first
              request to a proxied load balancer, a cookie is generated, encoding information
              of which origin the request will be forwarded to. Subsequent requests, by the
              same client to the same load balancer, will be sent to the origin server the
              cookie encodes, for the duration of the cookie and as long as the origin server
              remains healthy. If the cookie has expired or the origin server is unhealthy,
              then a new origin server is calculated and used. - `"ip_cookie"`: Behaves the
              same as `"cookie"` except the initial origin selection is stable and based on
              the client's ip address. - `"header"`: On the first request to a proxied load
              balancer, a session key based on the configured HTTP headers (see
              `session_affinity_attributes.headers`) is generated, encoding the request
              headers used for storing in the load balancer session state which origin the
              request will be forwarded to. Subsequent requests to the load balancer with the
              same headers will be sent to the same origin server, for the duration of the
              session and as long as the origin server remains healthy. If the session has
              been idle for the duration of `session_affinity_ttl` seconds or the origin
              server is unhealthy, then a new origin server is calculated and used. See
              `headers` in `session_affinity_attributes` for additional required
              configuration.

          session_affinity_attributes: Configures attributes for session affinity.

          session_affinity_ttl: Time, in seconds, until a client's session expires after being created. Once the
              expiry time has been reached, subsequent requests may get sent to a different
              origin server. The accepted ranges per `session_affinity` policy are: -
              `"cookie"` / `"ip_cookie"`: The current default of 23 hours will be used unless
              explicitly set. The accepted range of values is between [1800, 604800]. -
              `"header"`: The current default of 1800 seconds will be used unless explicitly
              set. The accepted range of values is between [30, 3600]. Note: With session
              affinity by header, sessions only expire after they haven't been used for the
              number of seconds specified.

          steering_policy: Steering Policy for this load balancer.

              - `"off"`: Use `default_pools`.
              - `"geo"`: Use `region_pools`/`country_pools`/`pop_pools`. For non-proxied
                requests, the country for `country_pools` is determined by
                `location_strategy`.
              - `"random"`: Select a pool randomly.
              - `"dynamic_latency"`: Use round trip time to select the closest pool in
                default_pools (requires pool health checks).
              - `"proximity"`: Use the pools' latitude and longitude to select the closest
                pool using the Cloudflare PoP location for proxied requests or the location
                determined by `location_strategy` for non-proxied requests.
              - `"least_outstanding_requests"`: Select a pool by taking into consideration
                `random_steering` weights, as well as each pool's number of outstanding
                requests. Pools with more pending requests are weighted proportionately less
                relative to others.
              - `"least_connections"`: Select a pool by taking into consideration
                `random_steering` weights, as well as each pool's number of open connections.
                Pools with more open connections are weighted proportionately less relative to
                others. Supported for HTTP/1 and HTTP/2 connections.
              - `""`: Will map to `"geo"` if you use
                `region_pools`/`country_pools`/`pop_pools` otherwise `"off"`.

          ttl: Time to live (TTL) of the DNS entry for the IP address returned by this load
              balancer. This only applies to gray-clouded (unproxied) load balancers.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not load_balancer_id:
            raise ValueError(f"Expected a non-empty value for `load_balancer_id` but received {load_balancer_id!r}")
        return self._put(
            path_template(
                "/zones/{zone_id}/load_balancers/{load_balancer_id}", zone_id=zone_id, load_balancer_id=load_balancer_id
            ),
            body=maybe_transform(
                {
                    "default_pools": default_pools,
                    "fallback_pool": fallback_pool,
                    "name": name,
                    "adaptive_routing": adaptive_routing,
                    "country_pools": country_pools,
                    "description": description,
                    "enabled": enabled,
                    "location_strategy": location_strategy,
                    "networks": networks,
                    "pop_pools": pop_pools,
                    "proxied": proxied,
                    "random_steering": random_steering,
                    "region_pools": region_pools,
                    "rules": rules,
                    "session_affinity": session_affinity,
                    "session_affinity_attributes": session_affinity_attributes,
                    "session_affinity_ttl": session_affinity_ttl,
                    "steering_policy": steering_policy,
                    "ttl": ttl,
                },
                load_balancer_update_params.LoadBalancerUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[LoadBalancer]._unwrapper,
            ),
            cast_to=cast(Type[LoadBalancer], ResultWrapper[LoadBalancer]),
        )

    def list(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[LoadBalancer]:
        """
        List configured load balancers.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/load_balancers", zone_id=zone_id),
            page=SyncSinglePage[LoadBalancer],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=LoadBalancer,
        )

    def delete(
        self,
        load_balancer_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LoadBalancerDeleteResponse:
        """
        Delete a configured load balancer.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not load_balancer_id:
            raise ValueError(f"Expected a non-empty value for `load_balancer_id` but received {load_balancer_id!r}")
        return self._delete(
            path_template(
                "/zones/{zone_id}/load_balancers/{load_balancer_id}", zone_id=zone_id, load_balancer_id=load_balancer_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[LoadBalancerDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[LoadBalancerDeleteResponse], ResultWrapper[LoadBalancerDeleteResponse]),
        )

    def edit(
        self,
        load_balancer_id: str,
        *,
        zone_id: str,
        adaptive_routing: AdaptiveRoutingParam | Omit = omit,
        country_pools: Dict[str, SequenceNotStr[str]] | Omit = omit,
        default_pools: SequenceNotStr[DefaultPools] | Omit = omit,
        description: str | Omit = omit,
        enabled: bool | Omit = omit,
        fallback_pool: str | Omit = omit,
        location_strategy: LocationStrategyParam | Omit = omit,
        name: str | Omit = omit,
        pop_pools: Dict[str, SequenceNotStr[str]] | Omit = omit,
        proxied: bool | Omit = omit,
        random_steering: RandomSteeringParam | Omit = omit,
        region_pools: Dict[str, SequenceNotStr[str]] | Omit = omit,
        rules: Iterable[RulesParam] | Omit = omit,
        session_affinity: SessionAffinity | Omit = omit,
        session_affinity_attributes: SessionAffinityAttributesParam | Omit = omit,
        session_affinity_ttl: float | Omit = omit,
        steering_policy: SteeringPolicy | Omit = omit,
        ttl: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LoadBalancer:
        """
        Apply changes to an existing load balancer, overwriting the supplied properties.

        Args:
          adaptive_routing: Controls features that modify the routing of requests to pools and origins in
              response to dynamic conditions, such as during the interval between active
              health monitoring requests. For example, zero-downtime failover occurs
              immediately when an origin becomes unavailable due to HTTP 521, 522, or 523
              response codes. If there is another healthy origin in the same pool, the request
              is retried once against this alternate origin.

          country_pools: A mapping of country codes to a list of pool IDs (ordered by their failover
              priority) for the given country. Any country not explicitly defined will fall
              back to using the corresponding region_pool mapping if it exists else to
              default_pools.

          default_pools: A list of pool IDs ordered by their failover priority. Pools defined here are
              used by default, or when region_pools are not configured for a given region.

          description: Object description.

          enabled: Whether to enable (the default) this load balancer.

          fallback_pool: The pool ID to use when all other pools are detected as unhealthy.

          location_strategy: Controls location-based steering for non-proxied requests. See `steering_policy`
              to learn how steering is affected.

          name: The DNS hostname to associate with your Load Balancer. If this hostname already
              exists as a DNS record in Cloudflare's DNS, the Load Balancer will take
              precedence and the DNS record will not be used.

          pop_pools: Enterprise only: A mapping of Cloudflare PoP identifiers to a list of pool IDs
              (ordered by their failover priority) for the PoP (datacenter). Any PoPs not
              explicitly defined will fall back to using the corresponding country_pool, then
              region_pool mapping if it exists else to default_pools.

          proxied: Whether the hostname should be gray clouded (false) or orange clouded (true).

          random_steering: Configures pool weights.

              - `steering_policy="random"`: A random pool is selected with probability
                proportional to pool weights.
              - `steering_policy="least_outstanding_requests"`: Use pool weights to scale each
                pool's outstanding requests.
              - `steering_policy="least_connections"`: Use pool weights to scale each pool's
                open connections.

          region_pools: A mapping of region codes to a list of pool IDs (ordered by their failover
              priority) for the given region. Any regions not explicitly defined will fall
              back to using default_pools.

          rules: BETA Field Not General Access: A list of rules for this load balancer to
              execute.

          session_affinity: Specifies the type of session affinity the load balancer should use unless
              specified as `"none"`. The supported types are: - `"cookie"`: On the first
              request to a proxied load balancer, a cookie is generated, encoding information
              of which origin the request will be forwarded to. Subsequent requests, by the
              same client to the same load balancer, will be sent to the origin server the
              cookie encodes, for the duration of the cookie and as long as the origin server
              remains healthy. If the cookie has expired or the origin server is unhealthy,
              then a new origin server is calculated and used. - `"ip_cookie"`: Behaves the
              same as `"cookie"` except the initial origin selection is stable and based on
              the client's ip address. - `"header"`: On the first request to a proxied load
              balancer, a session key based on the configured HTTP headers (see
              `session_affinity_attributes.headers`) is generated, encoding the request
              headers used for storing in the load balancer session state which origin the
              request will be forwarded to. Subsequent requests to the load balancer with the
              same headers will be sent to the same origin server, for the duration of the
              session and as long as the origin server remains healthy. If the session has
              been idle for the duration of `session_affinity_ttl` seconds or the origin
              server is unhealthy, then a new origin server is calculated and used. See
              `headers` in `session_affinity_attributes` for additional required
              configuration.

          session_affinity_attributes: Configures attributes for session affinity.

          session_affinity_ttl: Time, in seconds, until a client's session expires after being created. Once the
              expiry time has been reached, subsequent requests may get sent to a different
              origin server. The accepted ranges per `session_affinity` policy are: -
              `"cookie"` / `"ip_cookie"`: The current default of 23 hours will be used unless
              explicitly set. The accepted range of values is between [1800, 604800]. -
              `"header"`: The current default of 1800 seconds will be used unless explicitly
              set. The accepted range of values is between [30, 3600]. Note: With session
              affinity by header, sessions only expire after they haven't been used for the
              number of seconds specified.

          steering_policy: Steering Policy for this load balancer.

              - `"off"`: Use `default_pools`.
              - `"geo"`: Use `region_pools`/`country_pools`/`pop_pools`. For non-proxied
                requests, the country for `country_pools` is determined by
                `location_strategy`.
              - `"random"`: Select a pool randomly.
              - `"dynamic_latency"`: Use round trip time to select the closest pool in
                default_pools (requires pool health checks).
              - `"proximity"`: Use the pools' latitude and longitude to select the closest
                pool using the Cloudflare PoP location for proxied requests or the location
                determined by `location_strategy` for non-proxied requests.
              - `"least_outstanding_requests"`: Select a pool by taking into consideration
                `random_steering` weights, as well as each pool's number of outstanding
                requests. Pools with more pending requests are weighted proportionately less
                relative to others.
              - `"least_connections"`: Select a pool by taking into consideration
                `random_steering` weights, as well as each pool's number of open connections.
                Pools with more open connections are weighted proportionately less relative to
                others. Supported for HTTP/1 and HTTP/2 connections.
              - `""`: Will map to `"geo"` if you use
                `region_pools`/`country_pools`/`pop_pools` otherwise `"off"`.

          ttl: Time to live (TTL) of the DNS entry for the IP address returned by this load
              balancer. This only applies to gray-clouded (unproxied) load balancers.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not load_balancer_id:
            raise ValueError(f"Expected a non-empty value for `load_balancer_id` but received {load_balancer_id!r}")
        return self._patch(
            path_template(
                "/zones/{zone_id}/load_balancers/{load_balancer_id}", zone_id=zone_id, load_balancer_id=load_balancer_id
            ),
            body=maybe_transform(
                {
                    "adaptive_routing": adaptive_routing,
                    "country_pools": country_pools,
                    "default_pools": default_pools,
                    "description": description,
                    "enabled": enabled,
                    "fallback_pool": fallback_pool,
                    "location_strategy": location_strategy,
                    "name": name,
                    "pop_pools": pop_pools,
                    "proxied": proxied,
                    "random_steering": random_steering,
                    "region_pools": region_pools,
                    "rules": rules,
                    "session_affinity": session_affinity,
                    "session_affinity_attributes": session_affinity_attributes,
                    "session_affinity_ttl": session_affinity_ttl,
                    "steering_policy": steering_policy,
                    "ttl": ttl,
                },
                load_balancer_edit_params.LoadBalancerEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[LoadBalancer]._unwrapper,
            ),
            cast_to=cast(Type[LoadBalancer], ResultWrapper[LoadBalancer]),
        )

    def get(
        self,
        load_balancer_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LoadBalancer:
        """
        Fetch a single configured load balancer.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not load_balancer_id:
            raise ValueError(f"Expected a non-empty value for `load_balancer_id` but received {load_balancer_id!r}")
        return self._get(
            path_template(
                "/zones/{zone_id}/load_balancers/{load_balancer_id}", zone_id=zone_id, load_balancer_id=load_balancer_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[LoadBalancer]._unwrapper,
            ),
            cast_to=cast(Type[LoadBalancer], ResultWrapper[LoadBalancer]),
        )


class AsyncLoadBalancersResource(AsyncAPIResource):
    @cached_property
    def monitors(self) -> AsyncMonitorsResource:
        return AsyncMonitorsResource(self._client)

    @cached_property
    def monitor_groups(self) -> AsyncMonitorGroupsResource:
        return AsyncMonitorGroupsResource(self._client)

    @cached_property
    def pools(self) -> AsyncPoolsResource:
        return AsyncPoolsResource(self._client)

    @cached_property
    def previews(self) -> AsyncPreviewsResource:
        return AsyncPreviewsResource(self._client)

    @cached_property
    def regions(self) -> AsyncRegionsResource:
        return AsyncRegionsResource(self._client)

    @cached_property
    def searches(self) -> AsyncSearchesResource:
        return AsyncSearchesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncLoadBalancersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLoadBalancersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLoadBalancersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncLoadBalancersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        zone_id: str,
        default_pools: SequenceNotStr[DefaultPools],
        fallback_pool: str,
        name: str,
        adaptive_routing: AdaptiveRoutingParam | Omit = omit,
        country_pools: Dict[str, SequenceNotStr[str]] | Omit = omit,
        description: str | Omit = omit,
        location_strategy: LocationStrategyParam | Omit = omit,
        networks: SequenceNotStr[str] | Omit = omit,
        pop_pools: Dict[str, SequenceNotStr[str]] | Omit = omit,
        proxied: bool | Omit = omit,
        random_steering: RandomSteeringParam | Omit = omit,
        region_pools: Dict[str, SequenceNotStr[str]] | Omit = omit,
        rules: Iterable[RulesParam] | Omit = omit,
        session_affinity: SessionAffinity | Omit = omit,
        session_affinity_attributes: SessionAffinityAttributesParam | Omit = omit,
        session_affinity_ttl: float | Omit = omit,
        steering_policy: SteeringPolicy | Omit = omit,
        ttl: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LoadBalancer:
        """
        Create a new load balancer.

        Args:
          default_pools: A list of pool IDs ordered by their failover priority. Pools defined here are
              used by default, or when region_pools are not configured for a given region.

          fallback_pool: The pool ID to use when all other pools are detected as unhealthy.

          name: The DNS hostname to associate with your Load Balancer. If this hostname already
              exists as a DNS record in Cloudflare's DNS, the Load Balancer will take
              precedence and the DNS record will not be used.

          adaptive_routing: Controls features that modify the routing of requests to pools and origins in
              response to dynamic conditions, such as during the interval between active
              health monitoring requests. For example, zero-downtime failover occurs
              immediately when an origin becomes unavailable due to HTTP 521, 522, or 523
              response codes. If there is another healthy origin in the same pool, the request
              is retried once against this alternate origin.

          country_pools: A mapping of country codes to a list of pool IDs (ordered by their failover
              priority) for the given country. Any country not explicitly defined will fall
              back to using the corresponding region_pool mapping if it exists else to
              default_pools.

          description: Object description.

          location_strategy: Controls location-based steering for non-proxied requests. See `steering_policy`
              to learn how steering is affected.

          networks: List of networks where Load Balancer or Pool is enabled.

          pop_pools: Enterprise only: A mapping of Cloudflare PoP identifiers to a list of pool IDs
              (ordered by their failover priority) for the PoP (datacenter). Any PoPs not
              explicitly defined will fall back to using the corresponding country_pool, then
              region_pool mapping if it exists else to default_pools.

          proxied: Whether the hostname should be gray clouded (false) or orange clouded (true).

          random_steering: Configures pool weights.

              - `steering_policy="random"`: A random pool is selected with probability
                proportional to pool weights.
              - `steering_policy="least_outstanding_requests"`: Use pool weights to scale each
                pool's outstanding requests.
              - `steering_policy="least_connections"`: Use pool weights to scale each pool's
                open connections.

          region_pools: A mapping of region codes to a list of pool IDs (ordered by their failover
              priority) for the given region. Any regions not explicitly defined will fall
              back to using default_pools.

          rules: BETA Field Not General Access: A list of rules for this load balancer to
              execute.

          session_affinity: Specifies the type of session affinity the load balancer should use unless
              specified as `"none"`. The supported types are: - `"cookie"`: On the first
              request to a proxied load balancer, a cookie is generated, encoding information
              of which origin the request will be forwarded to. Subsequent requests, by the
              same client to the same load balancer, will be sent to the origin server the
              cookie encodes, for the duration of the cookie and as long as the origin server
              remains healthy. If the cookie has expired or the origin server is unhealthy,
              then a new origin server is calculated and used. - `"ip_cookie"`: Behaves the
              same as `"cookie"` except the initial origin selection is stable and based on
              the client's ip address. - `"header"`: On the first request to a proxied load
              balancer, a session key based on the configured HTTP headers (see
              `session_affinity_attributes.headers`) is generated, encoding the request
              headers used for storing in the load balancer session state which origin the
              request will be forwarded to. Subsequent requests to the load balancer with the
              same headers will be sent to the same origin server, for the duration of the
              session and as long as the origin server remains healthy. If the session has
              been idle for the duration of `session_affinity_ttl` seconds or the origin
              server is unhealthy, then a new origin server is calculated and used. See
              `headers` in `session_affinity_attributes` for additional required
              configuration.

          session_affinity_attributes: Configures attributes for session affinity.

          session_affinity_ttl: Time, in seconds, until a client's session expires after being created. Once the
              expiry time has been reached, subsequent requests may get sent to a different
              origin server. The accepted ranges per `session_affinity` policy are: -
              `"cookie"` / `"ip_cookie"`: The current default of 23 hours will be used unless
              explicitly set. The accepted range of values is between [1800, 604800]. -
              `"header"`: The current default of 1800 seconds will be used unless explicitly
              set. The accepted range of values is between [30, 3600]. Note: With session
              affinity by header, sessions only expire after they haven't been used for the
              number of seconds specified.

          steering_policy: Steering Policy for this load balancer.

              - `"off"`: Use `default_pools`.
              - `"geo"`: Use `region_pools`/`country_pools`/`pop_pools`. For non-proxied
                requests, the country for `country_pools` is determined by
                `location_strategy`.
              - `"random"`: Select a pool randomly.
              - `"dynamic_latency"`: Use round trip time to select the closest pool in
                default_pools (requires pool health checks).
              - `"proximity"`: Use the pools' latitude and longitude to select the closest
                pool using the Cloudflare PoP location for proxied requests or the location
                determined by `location_strategy` for non-proxied requests.
              - `"least_outstanding_requests"`: Select a pool by taking into consideration
                `random_steering` weights, as well as each pool's number of outstanding
                requests. Pools with more pending requests are weighted proportionately less
                relative to others.
              - `"least_connections"`: Select a pool by taking into consideration
                `random_steering` weights, as well as each pool's number of open connections.
                Pools with more open connections are weighted proportionately less relative to
                others. Supported for HTTP/1 and HTTP/2 connections.
              - `""`: Will map to `"geo"` if you use
                `region_pools`/`country_pools`/`pop_pools` otherwise `"off"`.

          ttl: Time to live (TTL) of the DNS entry for the IP address returned by this load
              balancer. This only applies to gray-clouded (unproxied) load balancers.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/load_balancers", zone_id=zone_id),
            body=await async_maybe_transform(
                {
                    "default_pools": default_pools,
                    "fallback_pool": fallback_pool,
                    "name": name,
                    "adaptive_routing": adaptive_routing,
                    "country_pools": country_pools,
                    "description": description,
                    "location_strategy": location_strategy,
                    "networks": networks,
                    "pop_pools": pop_pools,
                    "proxied": proxied,
                    "random_steering": random_steering,
                    "region_pools": region_pools,
                    "rules": rules,
                    "session_affinity": session_affinity,
                    "session_affinity_attributes": session_affinity_attributes,
                    "session_affinity_ttl": session_affinity_ttl,
                    "steering_policy": steering_policy,
                    "ttl": ttl,
                },
                load_balancer_create_params.LoadBalancerCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[LoadBalancer]._unwrapper,
            ),
            cast_to=cast(Type[LoadBalancer], ResultWrapper[LoadBalancer]),
        )

    async def update(
        self,
        load_balancer_id: str,
        *,
        zone_id: str,
        default_pools: SequenceNotStr[DefaultPools],
        fallback_pool: str,
        name: str,
        adaptive_routing: AdaptiveRoutingParam | Omit = omit,
        country_pools: Dict[str, SequenceNotStr[str]] | Omit = omit,
        description: str | Omit = omit,
        enabled: bool | Omit = omit,
        location_strategy: LocationStrategyParam | Omit = omit,
        networks: SequenceNotStr[str] | Omit = omit,
        pop_pools: Dict[str, SequenceNotStr[str]] | Omit = omit,
        proxied: bool | Omit = omit,
        random_steering: RandomSteeringParam | Omit = omit,
        region_pools: Dict[str, SequenceNotStr[str]] | Omit = omit,
        rules: Iterable[RulesParam] | Omit = omit,
        session_affinity: SessionAffinity | Omit = omit,
        session_affinity_attributes: SessionAffinityAttributesParam | Omit = omit,
        session_affinity_ttl: float | Omit = omit,
        steering_policy: SteeringPolicy | Omit = omit,
        ttl: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LoadBalancer:
        """
        Update a configured load balancer.

        Args:
          default_pools: A list of pool IDs ordered by their failover priority. Pools defined here are
              used by default, or when region_pools are not configured for a given region.

          fallback_pool: The pool ID to use when all other pools are detected as unhealthy.

          name: The DNS hostname to associate with your Load Balancer. If this hostname already
              exists as a DNS record in Cloudflare's DNS, the Load Balancer will take
              precedence and the DNS record will not be used.

          adaptive_routing: Controls features that modify the routing of requests to pools and origins in
              response to dynamic conditions, such as during the interval between active
              health monitoring requests. For example, zero-downtime failover occurs
              immediately when an origin becomes unavailable due to HTTP 521, 522, or 523
              response codes. If there is another healthy origin in the same pool, the request
              is retried once against this alternate origin.

          country_pools: A mapping of country codes to a list of pool IDs (ordered by their failover
              priority) for the given country. Any country not explicitly defined will fall
              back to using the corresponding region_pool mapping if it exists else to
              default_pools.

          description: Object description.

          enabled: Whether to enable (the default) this load balancer.

          location_strategy: Controls location-based steering for non-proxied requests. See `steering_policy`
              to learn how steering is affected.

          networks: List of networks where Load Balancer or Pool is enabled.

          pop_pools: Enterprise only: A mapping of Cloudflare PoP identifiers to a list of pool IDs
              (ordered by their failover priority) for the PoP (datacenter). Any PoPs not
              explicitly defined will fall back to using the corresponding country_pool, then
              region_pool mapping if it exists else to default_pools.

          proxied: Whether the hostname should be gray clouded (false) or orange clouded (true).

          random_steering: Configures pool weights.

              - `steering_policy="random"`: A random pool is selected with probability
                proportional to pool weights.
              - `steering_policy="least_outstanding_requests"`: Use pool weights to scale each
                pool's outstanding requests.
              - `steering_policy="least_connections"`: Use pool weights to scale each pool's
                open connections.

          region_pools: A mapping of region codes to a list of pool IDs (ordered by their failover
              priority) for the given region. Any regions not explicitly defined will fall
              back to using default_pools.

          rules: BETA Field Not General Access: A list of rules for this load balancer to
              execute.

          session_affinity: Specifies the type of session affinity the load balancer should use unless
              specified as `"none"`. The supported types are: - `"cookie"`: On the first
              request to a proxied load balancer, a cookie is generated, encoding information
              of which origin the request will be forwarded to. Subsequent requests, by the
              same client to the same load balancer, will be sent to the origin server the
              cookie encodes, for the duration of the cookie and as long as the origin server
              remains healthy. If the cookie has expired or the origin server is unhealthy,
              then a new origin server is calculated and used. - `"ip_cookie"`: Behaves the
              same as `"cookie"` except the initial origin selection is stable and based on
              the client's ip address. - `"header"`: On the first request to a proxied load
              balancer, a session key based on the configured HTTP headers (see
              `session_affinity_attributes.headers`) is generated, encoding the request
              headers used for storing in the load balancer session state which origin the
              request will be forwarded to. Subsequent requests to the load balancer with the
              same headers will be sent to the same origin server, for the duration of the
              session and as long as the origin server remains healthy. If the session has
              been idle for the duration of `session_affinity_ttl` seconds or the origin
              server is unhealthy, then a new origin server is calculated and used. See
              `headers` in `session_affinity_attributes` for additional required
              configuration.

          session_affinity_attributes: Configures attributes for session affinity.

          session_affinity_ttl: Time, in seconds, until a client's session expires after being created. Once the
              expiry time has been reached, subsequent requests may get sent to a different
              origin server. The accepted ranges per `session_affinity` policy are: -
              `"cookie"` / `"ip_cookie"`: The current default of 23 hours will be used unless
              explicitly set. The accepted range of values is between [1800, 604800]. -
              `"header"`: The current default of 1800 seconds will be used unless explicitly
              set. The accepted range of values is between [30, 3600]. Note: With session
              affinity by header, sessions only expire after they haven't been used for the
              number of seconds specified.

          steering_policy: Steering Policy for this load balancer.

              - `"off"`: Use `default_pools`.
              - `"geo"`: Use `region_pools`/`country_pools`/`pop_pools`. For non-proxied
                requests, the country for `country_pools` is determined by
                `location_strategy`.
              - `"random"`: Select a pool randomly.
              - `"dynamic_latency"`: Use round trip time to select the closest pool in
                default_pools (requires pool health checks).
              - `"proximity"`: Use the pools' latitude and longitude to select the closest
                pool using the Cloudflare PoP location for proxied requests or the location
                determined by `location_strategy` for non-proxied requests.
              - `"least_outstanding_requests"`: Select a pool by taking into consideration
                `random_steering` weights, as well as each pool's number of outstanding
                requests. Pools with more pending requests are weighted proportionately less
                relative to others.
              - `"least_connections"`: Select a pool by taking into consideration
                `random_steering` weights, as well as each pool's number of open connections.
                Pools with more open connections are weighted proportionately less relative to
                others. Supported for HTTP/1 and HTTP/2 connections.
              - `""`: Will map to `"geo"` if you use
                `region_pools`/`country_pools`/`pop_pools` otherwise `"off"`.

          ttl: Time to live (TTL) of the DNS entry for the IP address returned by this load
              balancer. This only applies to gray-clouded (unproxied) load balancers.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not load_balancer_id:
            raise ValueError(f"Expected a non-empty value for `load_balancer_id` but received {load_balancer_id!r}")
        return await self._put(
            path_template(
                "/zones/{zone_id}/load_balancers/{load_balancer_id}", zone_id=zone_id, load_balancer_id=load_balancer_id
            ),
            body=await async_maybe_transform(
                {
                    "default_pools": default_pools,
                    "fallback_pool": fallback_pool,
                    "name": name,
                    "adaptive_routing": adaptive_routing,
                    "country_pools": country_pools,
                    "description": description,
                    "enabled": enabled,
                    "location_strategy": location_strategy,
                    "networks": networks,
                    "pop_pools": pop_pools,
                    "proxied": proxied,
                    "random_steering": random_steering,
                    "region_pools": region_pools,
                    "rules": rules,
                    "session_affinity": session_affinity,
                    "session_affinity_attributes": session_affinity_attributes,
                    "session_affinity_ttl": session_affinity_ttl,
                    "steering_policy": steering_policy,
                    "ttl": ttl,
                },
                load_balancer_update_params.LoadBalancerUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[LoadBalancer]._unwrapper,
            ),
            cast_to=cast(Type[LoadBalancer], ResultWrapper[LoadBalancer]),
        )

    def list(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[LoadBalancer, AsyncSinglePage[LoadBalancer]]:
        """
        List configured load balancers.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/load_balancers", zone_id=zone_id),
            page=AsyncSinglePage[LoadBalancer],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=LoadBalancer,
        )

    async def delete(
        self,
        load_balancer_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LoadBalancerDeleteResponse:
        """
        Delete a configured load balancer.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not load_balancer_id:
            raise ValueError(f"Expected a non-empty value for `load_balancer_id` but received {load_balancer_id!r}")
        return await self._delete(
            path_template(
                "/zones/{zone_id}/load_balancers/{load_balancer_id}", zone_id=zone_id, load_balancer_id=load_balancer_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[LoadBalancerDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[LoadBalancerDeleteResponse], ResultWrapper[LoadBalancerDeleteResponse]),
        )

    async def edit(
        self,
        load_balancer_id: str,
        *,
        zone_id: str,
        adaptive_routing: AdaptiveRoutingParam | Omit = omit,
        country_pools: Dict[str, SequenceNotStr[str]] | Omit = omit,
        default_pools: SequenceNotStr[DefaultPools] | Omit = omit,
        description: str | Omit = omit,
        enabled: bool | Omit = omit,
        fallback_pool: str | Omit = omit,
        location_strategy: LocationStrategyParam | Omit = omit,
        name: str | Omit = omit,
        pop_pools: Dict[str, SequenceNotStr[str]] | Omit = omit,
        proxied: bool | Omit = omit,
        random_steering: RandomSteeringParam | Omit = omit,
        region_pools: Dict[str, SequenceNotStr[str]] | Omit = omit,
        rules: Iterable[RulesParam] | Omit = omit,
        session_affinity: SessionAffinity | Omit = omit,
        session_affinity_attributes: SessionAffinityAttributesParam | Omit = omit,
        session_affinity_ttl: float | Omit = omit,
        steering_policy: SteeringPolicy | Omit = omit,
        ttl: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LoadBalancer:
        """
        Apply changes to an existing load balancer, overwriting the supplied properties.

        Args:
          adaptive_routing: Controls features that modify the routing of requests to pools and origins in
              response to dynamic conditions, such as during the interval between active
              health monitoring requests. For example, zero-downtime failover occurs
              immediately when an origin becomes unavailable due to HTTP 521, 522, or 523
              response codes. If there is another healthy origin in the same pool, the request
              is retried once against this alternate origin.

          country_pools: A mapping of country codes to a list of pool IDs (ordered by their failover
              priority) for the given country. Any country not explicitly defined will fall
              back to using the corresponding region_pool mapping if it exists else to
              default_pools.

          default_pools: A list of pool IDs ordered by their failover priority. Pools defined here are
              used by default, or when region_pools are not configured for a given region.

          description: Object description.

          enabled: Whether to enable (the default) this load balancer.

          fallback_pool: The pool ID to use when all other pools are detected as unhealthy.

          location_strategy: Controls location-based steering for non-proxied requests. See `steering_policy`
              to learn how steering is affected.

          name: The DNS hostname to associate with your Load Balancer. If this hostname already
              exists as a DNS record in Cloudflare's DNS, the Load Balancer will take
              precedence and the DNS record will not be used.

          pop_pools: Enterprise only: A mapping of Cloudflare PoP identifiers to a list of pool IDs
              (ordered by their failover priority) for the PoP (datacenter). Any PoPs not
              explicitly defined will fall back to using the corresponding country_pool, then
              region_pool mapping if it exists else to default_pools.

          proxied: Whether the hostname should be gray clouded (false) or orange clouded (true).

          random_steering: Configures pool weights.

              - `steering_policy="random"`: A random pool is selected with probability
                proportional to pool weights.
              - `steering_policy="least_outstanding_requests"`: Use pool weights to scale each
                pool's outstanding requests.
              - `steering_policy="least_connections"`: Use pool weights to scale each pool's
                open connections.

          region_pools: A mapping of region codes to a list of pool IDs (ordered by their failover
              priority) for the given region. Any regions not explicitly defined will fall
              back to using default_pools.

          rules: BETA Field Not General Access: A list of rules for this load balancer to
              execute.

          session_affinity: Specifies the type of session affinity the load balancer should use unless
              specified as `"none"`. The supported types are: - `"cookie"`: On the first
              request to a proxied load balancer, a cookie is generated, encoding information
              of which origin the request will be forwarded to. Subsequent requests, by the
              same client to the same load balancer, will be sent to the origin server the
              cookie encodes, for the duration of the cookie and as long as the origin server
              remains healthy. If the cookie has expired or the origin server is unhealthy,
              then a new origin server is calculated and used. - `"ip_cookie"`: Behaves the
              same as `"cookie"` except the initial origin selection is stable and based on
              the client's ip address. - `"header"`: On the first request to a proxied load
              balancer, a session key based on the configured HTTP headers (see
              `session_affinity_attributes.headers`) is generated, encoding the request
              headers used for storing in the load balancer session state which origin the
              request will be forwarded to. Subsequent requests to the load balancer with the
              same headers will be sent to the same origin server, for the duration of the
              session and as long as the origin server remains healthy. If the session has
              been idle for the duration of `session_affinity_ttl` seconds or the origin
              server is unhealthy, then a new origin server is calculated and used. See
              `headers` in `session_affinity_attributes` for additional required
              configuration.

          session_affinity_attributes: Configures attributes for session affinity.

          session_affinity_ttl: Time, in seconds, until a client's session expires after being created. Once the
              expiry time has been reached, subsequent requests may get sent to a different
              origin server. The accepted ranges per `session_affinity` policy are: -
              `"cookie"` / `"ip_cookie"`: The current default of 23 hours will be used unless
              explicitly set. The accepted range of values is between [1800, 604800]. -
              `"header"`: The current default of 1800 seconds will be used unless explicitly
              set. The accepted range of values is between [30, 3600]. Note: With session
              affinity by header, sessions only expire after they haven't been used for the
              number of seconds specified.

          steering_policy: Steering Policy for this load balancer.

              - `"off"`: Use `default_pools`.
              - `"geo"`: Use `region_pools`/`country_pools`/`pop_pools`. For non-proxied
                requests, the country for `country_pools` is determined by
                `location_strategy`.
              - `"random"`: Select a pool randomly.
              - `"dynamic_latency"`: Use round trip time to select the closest pool in
                default_pools (requires pool health checks).
              - `"proximity"`: Use the pools' latitude and longitude to select the closest
                pool using the Cloudflare PoP location for proxied requests or the location
                determined by `location_strategy` for non-proxied requests.
              - `"least_outstanding_requests"`: Select a pool by taking into consideration
                `random_steering` weights, as well as each pool's number of outstanding
                requests. Pools with more pending requests are weighted proportionately less
                relative to others.
              - `"least_connections"`: Select a pool by taking into consideration
                `random_steering` weights, as well as each pool's number of open connections.
                Pools with more open connections are weighted proportionately less relative to
                others. Supported for HTTP/1 and HTTP/2 connections.
              - `""`: Will map to `"geo"` if you use
                `region_pools`/`country_pools`/`pop_pools` otherwise `"off"`.

          ttl: Time to live (TTL) of the DNS entry for the IP address returned by this load
              balancer. This only applies to gray-clouded (unproxied) load balancers.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not load_balancer_id:
            raise ValueError(f"Expected a non-empty value for `load_balancer_id` but received {load_balancer_id!r}")
        return await self._patch(
            path_template(
                "/zones/{zone_id}/load_balancers/{load_balancer_id}", zone_id=zone_id, load_balancer_id=load_balancer_id
            ),
            body=await async_maybe_transform(
                {
                    "adaptive_routing": adaptive_routing,
                    "country_pools": country_pools,
                    "default_pools": default_pools,
                    "description": description,
                    "enabled": enabled,
                    "fallback_pool": fallback_pool,
                    "location_strategy": location_strategy,
                    "name": name,
                    "pop_pools": pop_pools,
                    "proxied": proxied,
                    "random_steering": random_steering,
                    "region_pools": region_pools,
                    "rules": rules,
                    "session_affinity": session_affinity,
                    "session_affinity_attributes": session_affinity_attributes,
                    "session_affinity_ttl": session_affinity_ttl,
                    "steering_policy": steering_policy,
                    "ttl": ttl,
                },
                load_balancer_edit_params.LoadBalancerEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[LoadBalancer]._unwrapper,
            ),
            cast_to=cast(Type[LoadBalancer], ResultWrapper[LoadBalancer]),
        )

    async def get(
        self,
        load_balancer_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LoadBalancer:
        """
        Fetch a single configured load balancer.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not load_balancer_id:
            raise ValueError(f"Expected a non-empty value for `load_balancer_id` but received {load_balancer_id!r}")
        return await self._get(
            path_template(
                "/zones/{zone_id}/load_balancers/{load_balancer_id}", zone_id=zone_id, load_balancer_id=load_balancer_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[LoadBalancer]._unwrapper,
            ),
            cast_to=cast(Type[LoadBalancer], ResultWrapper[LoadBalancer]),
        )


class LoadBalancersResourceWithRawResponse:
    def __init__(self, load_balancers: LoadBalancersResource) -> None:
        self._load_balancers = load_balancers

        self.create = to_raw_response_wrapper(
            load_balancers.create,
        )
        self.update = to_raw_response_wrapper(
            load_balancers.update,
        )
        self.list = to_raw_response_wrapper(
            load_balancers.list,
        )
        self.delete = to_raw_response_wrapper(
            load_balancers.delete,
        )
        self.edit = to_raw_response_wrapper(
            load_balancers.edit,
        )
        self.get = to_raw_response_wrapper(
            load_balancers.get,
        )

    @cached_property
    def monitors(self) -> MonitorsResourceWithRawResponse:
        return MonitorsResourceWithRawResponse(self._load_balancers.monitors)

    @cached_property
    def monitor_groups(self) -> MonitorGroupsResourceWithRawResponse:
        return MonitorGroupsResourceWithRawResponse(self._load_balancers.monitor_groups)

    @cached_property
    def pools(self) -> PoolsResourceWithRawResponse:
        return PoolsResourceWithRawResponse(self._load_balancers.pools)

    @cached_property
    def previews(self) -> PreviewsResourceWithRawResponse:
        return PreviewsResourceWithRawResponse(self._load_balancers.previews)

    @cached_property
    def regions(self) -> RegionsResourceWithRawResponse:
        return RegionsResourceWithRawResponse(self._load_balancers.regions)

    @cached_property
    def searches(self) -> SearchesResourceWithRawResponse:
        return SearchesResourceWithRawResponse(self._load_balancers.searches)


class AsyncLoadBalancersResourceWithRawResponse:
    def __init__(self, load_balancers: AsyncLoadBalancersResource) -> None:
        self._load_balancers = load_balancers

        self.create = async_to_raw_response_wrapper(
            load_balancers.create,
        )
        self.update = async_to_raw_response_wrapper(
            load_balancers.update,
        )
        self.list = async_to_raw_response_wrapper(
            load_balancers.list,
        )
        self.delete = async_to_raw_response_wrapper(
            load_balancers.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            load_balancers.edit,
        )
        self.get = async_to_raw_response_wrapper(
            load_balancers.get,
        )

    @cached_property
    def monitors(self) -> AsyncMonitorsResourceWithRawResponse:
        return AsyncMonitorsResourceWithRawResponse(self._load_balancers.monitors)

    @cached_property
    def monitor_groups(self) -> AsyncMonitorGroupsResourceWithRawResponse:
        return AsyncMonitorGroupsResourceWithRawResponse(self._load_balancers.monitor_groups)

    @cached_property
    def pools(self) -> AsyncPoolsResourceWithRawResponse:
        return AsyncPoolsResourceWithRawResponse(self._load_balancers.pools)

    @cached_property
    def previews(self) -> AsyncPreviewsResourceWithRawResponse:
        return AsyncPreviewsResourceWithRawResponse(self._load_balancers.previews)

    @cached_property
    def regions(self) -> AsyncRegionsResourceWithRawResponse:
        return AsyncRegionsResourceWithRawResponse(self._load_balancers.regions)

    @cached_property
    def searches(self) -> AsyncSearchesResourceWithRawResponse:
        return AsyncSearchesResourceWithRawResponse(self._load_balancers.searches)


class LoadBalancersResourceWithStreamingResponse:
    def __init__(self, load_balancers: LoadBalancersResource) -> None:
        self._load_balancers = load_balancers

        self.create = to_streamed_response_wrapper(
            load_balancers.create,
        )
        self.update = to_streamed_response_wrapper(
            load_balancers.update,
        )
        self.list = to_streamed_response_wrapper(
            load_balancers.list,
        )
        self.delete = to_streamed_response_wrapper(
            load_balancers.delete,
        )
        self.edit = to_streamed_response_wrapper(
            load_balancers.edit,
        )
        self.get = to_streamed_response_wrapper(
            load_balancers.get,
        )

    @cached_property
    def monitors(self) -> MonitorsResourceWithStreamingResponse:
        return MonitorsResourceWithStreamingResponse(self._load_balancers.monitors)

    @cached_property
    def monitor_groups(self) -> MonitorGroupsResourceWithStreamingResponse:
        return MonitorGroupsResourceWithStreamingResponse(self._load_balancers.monitor_groups)

    @cached_property
    def pools(self) -> PoolsResourceWithStreamingResponse:
        return PoolsResourceWithStreamingResponse(self._load_balancers.pools)

    @cached_property
    def previews(self) -> PreviewsResourceWithStreamingResponse:
        return PreviewsResourceWithStreamingResponse(self._load_balancers.previews)

    @cached_property
    def regions(self) -> RegionsResourceWithStreamingResponse:
        return RegionsResourceWithStreamingResponse(self._load_balancers.regions)

    @cached_property
    def searches(self) -> SearchesResourceWithStreamingResponse:
        return SearchesResourceWithStreamingResponse(self._load_balancers.searches)


class AsyncLoadBalancersResourceWithStreamingResponse:
    def __init__(self, load_balancers: AsyncLoadBalancersResource) -> None:
        self._load_balancers = load_balancers

        self.create = async_to_streamed_response_wrapper(
            load_balancers.create,
        )
        self.update = async_to_streamed_response_wrapper(
            load_balancers.update,
        )
        self.list = async_to_streamed_response_wrapper(
            load_balancers.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            load_balancers.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            load_balancers.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            load_balancers.get,
        )

    @cached_property
    def monitors(self) -> AsyncMonitorsResourceWithStreamingResponse:
        return AsyncMonitorsResourceWithStreamingResponse(self._load_balancers.monitors)

    @cached_property
    def monitor_groups(self) -> AsyncMonitorGroupsResourceWithStreamingResponse:
        return AsyncMonitorGroupsResourceWithStreamingResponse(self._load_balancers.monitor_groups)

    @cached_property
    def pools(self) -> AsyncPoolsResourceWithStreamingResponse:
        return AsyncPoolsResourceWithStreamingResponse(self._load_balancers.pools)

    @cached_property
    def previews(self) -> AsyncPreviewsResourceWithStreamingResponse:
        return AsyncPreviewsResourceWithStreamingResponse(self._load_balancers.previews)

    @cached_property
    def regions(self) -> AsyncRegionsResourceWithStreamingResponse:
        return AsyncRegionsResourceWithStreamingResponse(self._load_balancers.regions)

    @cached_property
    def searches(self) -> AsyncSearchesResourceWithStreamingResponse:
        return AsyncSearchesResourceWithStreamingResponse(self._load_balancers.searches)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["SubscriptionUpdateParams", "Destination"]


class SubscriptionUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """A Resource identifier."""

    destination: Destination
    """Destination configuration for the subscription"""

    enabled: bool
    """Whether the subscription is active"""

    events: SequenceNotStr[str]
    """List of event types this subscription handles"""

    name: str
    """Name of the subscription"""


class Destination(TypedDict, total=False):
    """Destination configuration for the subscription"""

    queue_id: Required[str]
    """ID of the target queue"""

    type: Required[Literal["queues.queue"]]
    """Type of destination"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import TypedDict

from ...._types import SequenceNotStr
from .allowed_headers import AllowedHeaders
from .allowed_methods import AllowedMethods
from .allowed_origins import AllowedOrigins

__all__ = ["CORSHeadersParam"]


class CORSHeadersParam(TypedDict, total=False):
    allow_all_headers: bool
    """Allows all HTTP request headers."""

    allow_all_methods: bool
    """Allows all HTTP request methods."""

    allow_all_origins: bool
    """Allows all origins."""

    allow_credentials: bool
    """
    When set to `true`, includes credentials (cookies, authorization headers, or TLS
    client certificates) with requests.
    """

    allowed_headers: SequenceNotStr[AllowedHeaders]
    """Allowed HTTP request headers."""

    allowed_methods: List[AllowedMethods]
    """Allowed HTTP request methods."""

    allowed_origins: SequenceNotStr[AllowedOrigins]
    """Allowed origins."""

    max_age: float
    """The maximum number of seconds the results of a preflight request can be cached."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ConnectorGetResponse", "Device"]


class Device(BaseModel):
    id: str

    serial_number: Optional[str] = None

    type: Optional[Literal["MANAGED", "LICENSED"]] = None


class ConnectorGetResponse(BaseModel):
    id: str

    activated: bool

    interrupt_window_days_of_week: List[
        Literal["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"]
    ]
    """Allowed days of the week for upgrades. Default is all days."""

    interrupt_window_duration_hours: float

    interrupt_window_embargo_dates: List[str]
    """List of dates (YYYY-MM-DD) when upgrades are blocked."""

    interrupt_window_hour_of_day: float

    last_updated: str

    notes: str

    timezone: str

    device: Optional[Device] = None

    last_heartbeat: Optional[str] = None

    last_seen_version: Optional[str] = None

    license_key: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .upload import (
    UploadResource,
    AsyncUploadResource,
    UploadResourceWithRawResponse,
    AsyncUploadResourceWithRawResponse,
    UploadResourceWithStreamingResponse,
    AsyncUploadResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["AssetsResource", "AsyncAssetsResource"]


class AssetsResource(SyncAPIResource):
    @cached_property
    def upload(self) -> UploadResource:
        return UploadResource(self._client)

    @cached_property
    def with_raw_response(self) -> AssetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AssetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AssetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AssetsResourceWithStreamingResponse(self)


class AsyncAssetsResource(AsyncAPIResource):
    @cached_property
    def upload(self) -> AsyncUploadResource:
        return AsyncUploadResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAssetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAssetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAssetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAssetsResourceWithStreamingResponse(self)


class AssetsResourceWithRawResponse:
    def __init__(self, assets: AssetsResource) -> None:
        self._assets = assets

    @cached_property
    def upload(self) -> UploadResourceWithRawResponse:
        return UploadResourceWithRawResponse(self._assets.upload)


class AsyncAssetsResourceWithRawResponse:
    def __init__(self, assets: AsyncAssetsResource) -> None:
        self._assets = assets

    @cached_property
    def upload(self) -> AsyncUploadResourceWithRawResponse:
        return AsyncUploadResourceWithRawResponse(self._assets.upload)


class AssetsResourceWithStreamingResponse:
    def __init__(self, assets: AssetsResource) -> None:
        self._assets = assets

    @cached_property
    def upload(self) -> UploadResourceWithStreamingResponse:
        return UploadResourceWithStreamingResponse(self._assets.upload)


class AsyncAssetsResourceWithStreamingResponse:
    def __init__(self, assets: AsyncAssetsResource) -> None:
        self._assets = assets

    @cached_property
    def upload(self) -> AsyncUploadResourceWithStreamingResponse:
        return AsyncUploadResourceWithStreamingResponse(self._assets.upload)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from ....._models import BaseModel

__all__ = ["DeviceListResponse", "Device"]


class Device(BaseModel):
    device_id: Optional[str] = FieldInfo(alias="deviceId", default=None)
    """Device identifier (UUID v4)"""

    device_name: Optional[str] = FieldInfo(alias="deviceName", default=None)
    """Device identifier (human readable)"""

    eligible: Optional[bool] = None
    """Whether the device is eligible for remote captures"""

    ineligible_reason: Optional[str] = FieldInfo(alias="ineligibleReason", default=None)
    """If the device is not eligible, the reason why."""

    person_email: Optional[str] = FieldInfo(alias="personEmail", default=None)
    """User contact email address"""

    platform: Optional[str] = None
    """Operating system"""

    registration_id: Optional[str] = FieldInfo(alias="registrationId", default=None)
    """Device registration identifier (UUID v4).

    On multi-user devices, this uniquely identifies a user's registration on the
    device.
    """

    status: Optional[str] = None
    """Network status"""

    timestamp: Optional[str] = None
    """Timestamp in ISO format"""

    version: Optional[str] = None
    """WARP client version"""


class DeviceListResponse(BaseModel):
    devices: Optional[List[Device]] = None
    """List of eligible devices"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

from .action_param import ActionParam
from .matcher_param import MatcherParam

__all__ = ["RuleCreateParams"]


class RuleCreateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    actions: Required[Iterable[ActionParam]]
    """List actions patterns."""

    matchers: Required[Iterable[MatcherParam]]
    """Matching patterns to forward to your actions."""

    enabled: Literal[True, False]
    """Routing rule status."""

    name: str
    """Routing rule name."""

    priority: float
    """Priority of the routing rule."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["EventNotificationUpdateParams", "Rule"]


class EventNotificationUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    bucket_name: Required[str]
    """Name of the bucket."""

    rules: Required[Iterable[Rule]]
    """Array of rules to drive notifications."""

    jurisdiction: Annotated[Literal["default", "eu", "fedramp"], PropertyInfo(alias="cf-r2-jurisdiction")]
    """Jurisdiction where objects in this bucket are guaranteed to be stored."""


class Rule(TypedDict, total=False):
    actions: Required[
        List[Literal["PutObject", "CopyObject", "DeleteObject", "CompleteMultipartUpload", "LifecycleDeletion"]]
    ]
    """Array of R2 object actions that will trigger notifications."""

    description: str
    """
    A description that can be used to identify the event notification rule after
    creation.
    """

    prefix: str
    """Notifications will be sent only for objects with this prefix."""

    suffix: str
    """Notifications will be sent only for objects with this suffix."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncSinglePage, AsyncSinglePage
from ...types.calls import turn_create_params, turn_update_params
from ..._base_client import AsyncPaginator, make_request_options
from ...types.calls.turn_get_response import TURNGetResponse
from ...types.calls.turn_list_response import TURNListResponse
from ...types.calls.turn_create_response import TURNCreateResponse
from ...types.calls.turn_delete_response import TURNDeleteResponse
from ...types.calls.turn_update_response import TURNUpdateResponse

__all__ = ["TURNResource", "AsyncTURNResource"]


class TURNResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TURNResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TURNResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TURNResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TURNResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TURNCreateResponse]:
        """
        Creates a new Cloudflare Calls TURN key.

        Args:
          account_id: The account identifier tag.

          name: A short description of a TURN key, not shown to end users.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/calls/turn_keys", account_id=account_id),
            body=maybe_transform({"name": name}, turn_create_params.TURNCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TURNCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TURNCreateResponse]], ResultWrapper[TURNCreateResponse]),
        )

    def update(
        self,
        key_id: str,
        *,
        account_id: str,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TURNUpdateResponse]:
        """
        Edit details for a single TURN key.

        Args:
          account_id: The account identifier tag.

          key_id: A Cloudflare-generated unique identifier for a item.

          name: A short description of a TURN key, not shown to end users.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not key_id:
            raise ValueError(f"Expected a non-empty value for `key_id` but received {key_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/calls/turn_keys/{key_id}", account_id=account_id, key_id=key_id),
            body=maybe_transform({"name": name}, turn_update_params.TURNUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TURNUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TURNUpdateResponse]], ResultWrapper[TURNUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[TURNListResponse]:
        """
        Lists all TURN keys in the Cloudflare account

        Args:
          account_id: The account identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/calls/turn_keys", account_id=account_id),
            page=SyncSinglePage[TURNListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=TURNListResponse,
        )

    def delete(
        self,
        key_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TURNDeleteResponse]:
        """
        Deletes a TURN key from Cloudflare Calls

        Args:
          account_id: The account identifier tag.

          key_id: A Cloudflare-generated unique identifier for a item.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not key_id:
            raise ValueError(f"Expected a non-empty value for `key_id` but received {key_id!r}")
        return self._delete(
            path_template("/accounts/{account_id}/calls/turn_keys/{key_id}", account_id=account_id, key_id=key_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TURNDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TURNDeleteResponse]], ResultWrapper[TURNDeleteResponse]),
        )

    def get(
        self,
        key_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TURNGetResponse]:
        """
        Fetches details for a single TURN key.

        Args:
          account_id: The account identifier tag.

          key_id: A Cloudflare-generated unique identifier for a item.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not key_id:
            raise ValueError(f"Expected a non-empty value for `key_id` but received {key_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/calls/turn_keys/{key_id}", account_id=account_id, key_id=key_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TURNGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TURNGetResponse]], ResultWrapper[TURNGetResponse]),
        )


class AsyncTURNResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTURNResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTURNResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTURNResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTURNResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TURNCreateResponse]:
        """
        Creates a new Cloudflare Calls TURN key.

        Args:
          account_id: The account identifier tag.

          name: A short description of a TURN key, not shown to end users.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/calls/turn_keys", account_id=account_id),
            body=await async_maybe_transform({"name": name}, turn_create_params.TURNCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TURNCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TURNCreateResponse]], ResultWrapper[TURNCreateResponse]),
        )

    async def update(
        self,
        key_id: str,
        *,
        account_id: str,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TURNUpdateResponse]:
        """
        Edit details for a single TURN key.

        Args:
          account_id: The account identifier tag.

          key_id: A Cloudflare-generated unique identifier for a item.

          name: A short description of a TURN key, not shown to end users.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not key_id:
            raise ValueError(f"Expected a non-empty value for `key_id` but received {key_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/calls/turn_keys/{key_id}", account_id=account_id, key_id=key_id),
            body=await async_maybe_transform({"name": name}, turn_update_params.TURNUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TURNUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TURNUpdateResponse]], ResultWrapper[TURNUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[TURNListResponse, AsyncSinglePage[TURNListResponse]]:
        """
        Lists all TURN keys in the Cloudflare account

        Args:
          account_id: The account identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/calls/turn_keys", account_id=account_id),
            page=AsyncSinglePage[TURNListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=TURNListResponse,
        )

    async def delete(
        self,
        key_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TURNDeleteResponse]:
        """
        Deletes a TURN key from Cloudflare Calls

        Args:
          account_id: The account identifier tag.

          key_id: A Cloudflare-generated unique identifier for a item.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not key_id:
            raise ValueError(f"Expected a non-empty value for `key_id` but received {key_id!r}")
        return await self._delete(
            path_template("/accounts/{account_id}/calls/turn_keys/{key_id}", account_id=account_id, key_id=key_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TURNDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TURNDeleteResponse]], ResultWrapper[TURNDeleteResponse]),
        )

    async def get(
        self,
        key_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TURNGetResponse]:
        """
        Fetches details for a single TURN key.

        Args:
          account_id: The account identifier tag.

          key_id: A Cloudflare-generated unique identifier for a item.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not key_id:
            raise ValueError(f"Expected a non-empty value for `key_id` but received {key_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/calls/turn_keys/{key_id}", account_id=account_id, key_id=key_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TURNGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TURNGetResponse]], ResultWrapper[TURNGetResponse]),
        )


class TURNResourceWithRawResponse:
    def __init__(self, turn: TURNResource) -> None:
        self._turn = turn

        self.create = to_raw_response_wrapper(
            turn.create,
        )
        self.update = to_raw_response_wrapper(
            turn.update,
        )
        self.list = to_raw_response_wrapper(
            turn.list,
        )
        self.delete = to_raw_response_wrapper(
            turn.delete,
        )
        self.get = to_raw_response_wrapper(
            turn.get,
        )


class AsyncTURNResourceWithRawResponse:
    def __init__(self, turn: AsyncTURNResource) -> None:
        self._turn = turn

        self.create = async_to_raw_response_wrapper(
            turn.create,
        )
        self.update = async_to_raw_response_wrapper(
            turn.update,
        )
        self.list = async_to_raw_response_wrapper(
            turn.list,
        )
        self.delete = async_to_raw_response_wrapper(
            turn.delete,
        )
        self.get = async_to_raw_response_wrapper(
            turn.get,
        )


class TURNResourceWithStreamingResponse:
    def __init__(self, turn: TURNResource) -> None:
        self._turn = turn

        self.create = to_streamed_response_wrapper(
            turn.create,
        )
        self.update = to_streamed_response_wrapper(
            turn.update,
        )
        self.list = to_streamed_response_wrapper(
            turn.list,
        )
        self.delete = to_streamed_response_wrapper(
            turn.delete,
        )
        self.get = to_streamed_response_wrapper(
            turn.get,
        )


class AsyncTURNResourceWithStreamingResponse:
    def __init__(self, turn: AsyncTURNResource) -> None:
        self._turn = turn

        self.create = async_to_streamed_response_wrapper(
            turn.create,
        )
        self.update = async_to_streamed_response_wrapper(
            turn.update,
        )
        self.list = async_to_streamed_response_wrapper(
            turn.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            turn.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            turn.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from ...._utils import PropertyInfo

__all__ = [
    "LifecycleUpdateParams",
    "Rule",
    "RuleConditions",
    "RuleAbortMultipartUploadsTransition",
    "RuleAbortMultipartUploadsTransitionCondition",
    "RuleDeleteObjectsTransition",
    "RuleDeleteObjectsTransitionCondition",
    "RuleDeleteObjectsTransitionConditionR2LifecycleAgeCondition",
    "RuleDeleteObjectsTransitionConditionR2LifecycleDateCondition",
    "RuleStorageClassTransition",
    "RuleStorageClassTransitionCondition",
    "RuleStorageClassTransitionConditionR2LifecycleAgeCondition",
    "RuleStorageClassTransitionConditionR2LifecycleDateCondition",
]


class LifecycleUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    rules: Iterable[Rule]

    jurisdiction: Annotated[Literal["default", "eu", "fedramp"], PropertyInfo(alias="cf-r2-jurisdiction")]
    """Jurisdiction where objects in this bucket are guaranteed to be stored."""


class RuleConditions(TypedDict, total=False):
    """Conditions that apply to all transitions of this rule."""

    prefix: Required[str]
    """
    Transitions will only apply to objects/uploads in the bucket that start with the
    given prefix, an empty prefix can be provided to scope rule to all
    objects/uploads.
    """


class RuleAbortMultipartUploadsTransitionCondition(TypedDict, total=False):
    """
    Condition for lifecycle transitions to apply after an object reaches an age in seconds.
    """

    max_age: Required[Annotated[int, PropertyInfo(alias="maxAge")]]

    type: Required[Literal["Age"]]


class RuleAbortMultipartUploadsTransition(TypedDict, total=False):
    """Transition to abort ongoing multipart uploads."""

    condition: RuleAbortMultipartUploadsTransitionCondition
    """
    Condition for lifecycle transitions to apply after an object reaches an age in
    seconds.
    """


class RuleDeleteObjectsTransitionConditionR2LifecycleAgeCondition(TypedDict, total=False):
    """
    Condition for lifecycle transitions to apply after an object reaches an age in seconds.
    """

    max_age: Required[Annotated[int, PropertyInfo(alias="maxAge")]]

    type: Required[Literal["Age"]]


class RuleDeleteObjectsTransitionConditionR2LifecycleDateCondition(TypedDict, total=False):
    """Condition for lifecycle transitions to apply on a specific date."""

    date: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    type: Required[Literal["Date"]]


RuleDeleteObjectsTransitionCondition: TypeAlias = Union[
    RuleDeleteObjectsTransitionConditionR2LifecycleAgeCondition,
    RuleDeleteObjectsTransitionConditionR2LifecycleDateCondition,
]


class RuleDeleteObjectsTransition(TypedDict, total=False):
    """Transition to delete objects."""

    condition: RuleDeleteObjectsTransitionCondition
    """
    Condition for lifecycle transitions to apply after an object reaches an age in
    seconds.
    """


class RuleStorageClassTransitionConditionR2LifecycleAgeCondition(TypedDict, total=False):
    """
    Condition for lifecycle transitions to apply after an object reaches an age in seconds.
    """

    max_age: Required[Annotated[int, PropertyInfo(alias="maxAge")]]

    type: Required[Literal["Age"]]


class RuleStorageClassTransitionConditionR2LifecycleDateCondition(TypedDict, total=False):
    """Condition for lifecycle transitions to apply on a specific date."""

    date: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    type: Required[Literal["Date"]]


RuleStorageClassTransitionCondition: TypeAlias = Union[
    RuleStorageClassTransitionConditionR2LifecycleAgeCondition,
    RuleStorageClassTransitionConditionR2LifecycleDateCondition,
]


class RuleStorageClassTransition(TypedDict, total=False):
    condition: Required[RuleStorageClassTransitionCondition]
    """
    Condition for lifecycle transitions to apply after an object reaches an age in
    seconds.
    """

    storage_class: Required[Annotated[Literal["InfrequentAccess"], PropertyInfo(alias="storageClass")]]


class Rule(TypedDict, total=False):
    id: Required[str]
    """Unique identifier for this rule."""

    conditions: Required[RuleConditions]
    """Conditions that apply to all transitions of this rule."""

    enabled: Required[bool]
    """Whether or not this rule is in effect."""

    abort_multipart_uploads_transition: Annotated[
        RuleAbortMultipartUploadsTransition, PropertyInfo(alias="abortMultipartUploadsTransition")
    ]
    """Transition to abort ongoing multipart uploads."""

    delete_objects_transition: Annotated[RuleDeleteObjectsTransition, PropertyInfo(alias="deleteObjectsTransition")]
    """Transition to delete objects."""

    storage_class_transitions: Annotated[
        Iterable[RuleStorageClassTransition], PropertyInfo(alias="storageClassTransitions")
    ]
    """Transitions to change the storage class of objects."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import TypeAlias

__all__ = ["AbuseReportCreateResponse"]

AbuseReportCreateResponse: TypeAlias = str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .rules import (
    RulesResource,
    AsyncRulesResource,
    RulesResourceWithRawResponse,
    AsyncRulesResourceWithRawResponse,
    RulesResourceWithStreamingResponse,
    AsyncRulesResourceWithStreamingResponse,
)
from .waf.waf import (
    WAFResource,
    AsyncWAFResource,
    WAFResourceWithRawResponse,
    AsyncWAFResourceWithRawResponse,
    WAFResourceWithStreamingResponse,
    AsyncWAFResourceWithStreamingResponse,
)
from .ua_rules import (
    UARulesResource,
    AsyncUARulesResource,
    UARulesResourceWithRawResponse,
    AsyncUARulesResourceWithRawResponse,
    UARulesResourceWithStreamingResponse,
    AsyncUARulesResourceWithStreamingResponse,
)
from ..._compat import cached_property
from .lockdowns import (
    LockdownsResource,
    AsyncLockdownsResource,
    LockdownsResourceWithRawResponse,
    AsyncLockdownsResourceWithRawResponse,
    LockdownsResourceWithStreamingResponse,
    AsyncLockdownsResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from .access_rules import (
    AccessRulesResource,
    AsyncAccessRulesResource,
    AccessRulesResourceWithRawResponse,
    AsyncAccessRulesResourceWithRawResponse,
    AccessRulesResourceWithStreamingResponse,
    AsyncAccessRulesResourceWithStreamingResponse,
)

__all__ = ["FirewallResource", "AsyncFirewallResource"]


class FirewallResource(SyncAPIResource):
    @cached_property
    def lockdowns(self) -> LockdownsResource:
        return LockdownsResource(self._client)

    @cached_property
    def rules(self) -> RulesResource:
        return RulesResource(self._client)

    @cached_property
    def access_rules(self) -> AccessRulesResource:
        return AccessRulesResource(self._client)

    @cached_property
    def ua_rules(self) -> UARulesResource:
        return UARulesResource(self._client)

    @cached_property
    def waf(self) -> WAFResource:
        return WAFResource(self._client)

    @cached_property
    def with_raw_response(self) -> FirewallResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return FirewallResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FirewallResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return FirewallResourceWithStreamingResponse(self)


class AsyncFirewallResource(AsyncAPIResource):
    @cached_property
    def lockdowns(self) -> AsyncLockdownsResource:
        return AsyncLockdownsResource(self._client)

    @cached_property
    def rules(self) -> AsyncRulesResource:
        return AsyncRulesResource(self._client)

    @cached_property
    def access_rules(self) -> AsyncAccessRulesResource:
        return AsyncAccessRulesResource(self._client)

    @cached_property
    def ua_rules(self) -> AsyncUARulesResource:
        return AsyncUARulesResource(self._client)

    @cached_property
    def waf(self) -> AsyncWAFResource:
        return AsyncWAFResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncFirewallResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFirewallResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFirewallResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncFirewallResourceWithStreamingResponse(self)


class FirewallResourceWithRawResponse:
    def __init__(self, firewall: FirewallResource) -> None:
        self._firewall = firewall

    @cached_property
    def lockdowns(self) -> LockdownsResourceWithRawResponse:
        return LockdownsResourceWithRawResponse(self._firewall.lockdowns)

    @cached_property
    def rules(self) -> RulesResourceWithRawResponse:
        return RulesResourceWithRawResponse(self._firewall.rules)

    @cached_property
    def access_rules(self) -> AccessRulesResourceWithRawResponse:
        return AccessRulesResourceWithRawResponse(self._firewall.access_rules)

    @cached_property
    def ua_rules(self) -> UARulesResourceWithRawResponse:
        return UARulesResourceWithRawResponse(self._firewall.ua_rules)

    @cached_property
    def waf(self) -> WAFResourceWithRawResponse:
        return WAFResourceWithRawResponse(self._firewall.waf)


class AsyncFirewallResourceWithRawResponse:
    def __init__(self, firewall: AsyncFirewallResource) -> None:
        self._firewall = firewall

    @cached_property
    def lockdowns(self) -> AsyncLockdownsResourceWithRawResponse:
        return AsyncLockdownsResourceWithRawResponse(self._firewall.lockdowns)

    @cached_property
    def rules(self) -> AsyncRulesResourceWithRawResponse:
        return AsyncRulesResourceWithRawResponse(self._firewall.rules)

    @cached_property
    def access_rules(self) -> AsyncAccessRulesResourceWithRawResponse:
        return AsyncAccessRulesResourceWithRawResponse(self._firewall.access_rules)

    @cached_property
    def ua_rules(self) -> AsyncUARulesResourceWithRawResponse:
        return AsyncUARulesResourceWithRawResponse(self._firewall.ua_rules)

    @cached_property
    def waf(self) -> AsyncWAFResourceWithRawResponse:
        return AsyncWAFResourceWithRawResponse(self._firewall.waf)


class FirewallResourceWithStreamingResponse:
    def __init__(self, firewall: FirewallResource) -> None:
        self._firewall = firewall

    @cached_property
    def lockdowns(self) -> LockdownsResourceWithStreamingResponse:
        return LockdownsResourceWithStreamingResponse(self._firewall.lockdowns)

    @cached_property
    def rules(self) -> RulesResourceWithStreamingResponse:
        return RulesResourceWithStreamingResponse(self._firewall.rules)

    @cached_property
    def access_rules(self) -> AccessRulesResourceWithStreamingResponse:
        return AccessRulesResourceWithStreamingResponse(self._firewall.access_rules)

    @cached_property
    def ua_rules(self) -> UARulesResourceWithStreamingResponse:
        return UARulesResourceWithStreamingResponse(self._firewall.ua_rules)

    @cached_property
    def waf(self) -> WAFResourceWithStreamingResponse:
        return WAFResourceWithStreamingResponse(self._firewall.waf)


class AsyncFirewallResourceWithStreamingResponse:
    def __init__(self, firewall: AsyncFirewallResource) -> None:
        self._firewall = firewall

    @cached_property
    def lockdowns(self) -> AsyncLockdownsResourceWithStreamingResponse:
        return AsyncLockdownsResourceWithStreamingResponse(self._firewall.lockdowns)

    @cached_property
    def rules(self) -> AsyncRulesResourceWithStreamingResponse:
        return AsyncRulesResourceWithStreamingResponse(self._firewall.rules)

    @cached_property
    def access_rules(self) -> AsyncAccessRulesResourceWithStreamingResponse:
        return AsyncAccessRulesResourceWithStreamingResponse(self._firewall.access_rules)

    @cached_property
    def ua_rules(self) -> AsyncUARulesResourceWithStreamingResponse:
        return AsyncUARulesResourceWithStreamingResponse(self._firewall.ua_rules)

    @cached_property
    def waf(self) -> AsyncWAFResourceWithStreamingResponse:
        return AsyncWAFResourceWithStreamingResponse(self._firewall.waf)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["RocketLoaderParam"]


class RocketLoaderParam(TypedDict, total=False):
    id: Literal["rocket_loader"]
    """Turn on or off Rocket Loader in the Cloudflare Speed app."""

    value: Literal["on", "off"]
    """The status of Rocket Loader"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast
from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.zero_trust.networks.subnets import cloudflare_source_update_params
from .....types.zero_trust.networks.subnets.subnet import Subnet

__all__ = ["CloudflareSourceResource", "AsyncCloudflareSourceResource"]


class CloudflareSourceResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CloudflareSourceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return CloudflareSourceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CloudflareSourceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return CloudflareSourceResourceWithStreamingResponse(self)

    def update(
        self,
        address_family: Literal["v4", "v6"],
        *,
        account_id: str,
        comment: str | Omit = omit,
        name: str | Omit = omit,
        network: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Subnet:
        """
        Updates the Cloudflare Source subnet of the given address family

        Args:
          account_id: Cloudflare account ID

          address_family: IP address family, either `v4` (IPv4) or `v6` (IPv6)

          comment: An optional description of the subnet.

          name: A user-friendly name for the subnet.

          network: The private IPv4 or IPv6 range defining the subnet, in CIDR notation.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not address_family:
            raise ValueError(f"Expected a non-empty value for `address_family` but received {address_family!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/zerotrust/subnets/cloudflare_source/{address_family}",
                account_id=account_id,
                address_family=address_family,
            ),
            body=maybe_transform(
                {
                    "comment": comment,
                    "name": name,
                    "network": network,
                },
                cloudflare_source_update_params.CloudflareSourceUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Subnet]._unwrapper,
            ),
            cast_to=cast(Type[Subnet], ResultWrapper[Subnet]),
        )


class AsyncCloudflareSourceResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCloudflareSourceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCloudflareSourceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCloudflareSourceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncCloudflareSourceResourceWithStreamingResponse(self)

    async def update(
        self,
        address_family: Literal["v4", "v6"],
        *,
        account_id: str,
        comment: str | Omit = omit,
        name: str | Omit = omit,
        network: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Subnet:
        """
        Updates the Cloudflare Source subnet of the given address family

        Args:
          account_id: Cloudflare account ID

          address_family: IP address family, either `v4` (IPv4) or `v6` (IPv6)

          comment: An optional description of the subnet.

          name: A user-friendly name for the subnet.

          network: The private IPv4 or IPv6 range defining the subnet, in CIDR notation.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not address_family:
            raise ValueError(f"Expected a non-empty value for `address_family` but received {address_family!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/zerotrust/subnets/cloudflare_source/{address_family}",
                account_id=account_id,
                address_family=address_family,
            ),
            body=await async_maybe_transform(
                {
                    "comment": comment,
                    "name": name,
                    "network": network,
                },
                cloudflare_source_update_params.CloudflareSourceUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Subnet]._unwrapper,
            ),
            cast_to=cast(Type[Subnet], ResultWrapper[Subnet]),
        )


class CloudflareSourceResourceWithRawResponse:
    def __init__(self, cloudflare_source: CloudflareSourceResource) -> None:
        self._cloudflare_source = cloudflare_source

        self.update = to_raw_response_wrapper(
            cloudflare_source.update,
        )


class AsyncCloudflareSourceResourceWithRawResponse:
    def __init__(self, cloudflare_source: AsyncCloudflareSourceResource) -> None:
        self._cloudflare_source = cloudflare_source

        self.update = async_to_raw_response_wrapper(
            cloudflare_source.update,
        )


class CloudflareSourceResourceWithStreamingResponse:
    def __init__(self, cloudflare_source: CloudflareSourceResource) -> None:
        self._cloudflare_source = cloudflare_source

        self.update = to_streamed_response_wrapper(
            cloudflare_source.update,
        )


class AsyncCloudflareSourceResourceWithStreamingResponse:
    def __init__(self, cloudflare_source: AsyncCloudflareSourceResource) -> None:
        self._cloudflare_source = cloudflare_source

        self.update = async_to_streamed_response_wrapper(
            cloudflare_source.update,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["MitigationReviewParams", "Appeal"]


class MitigationReviewParams(TypedDict, total=False):
    account_id: Required[str]

    appeals: Required[Iterable[Appeal]]
    """List of mitigations to appeal."""


class Appeal(TypedDict, total=False):
    id: Required[str]
    """ID of the mitigation to appeal."""

    reason: Required[Literal["removed", "misclassified"]]
    """Reason why the customer is appealing."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .domains import (
    DomainsResource,
    AsyncDomainsResource,
    DomainsResourceWithRawResponse,
    AsyncDomainsResourceWithRawResponse,
    DomainsResourceWithStreamingResponse,
    AsyncDomainsResourceWithStreamingResponse,
)
from .settings import (
    SettingsResource,
    AsyncSettingsResource,
    SettingsResourceWithRawResponse,
    AsyncSettingsResourceWithRawResponse,
    SettingsResourceWithStreamingResponse,
    AsyncSettingsResourceWithStreamingResponse,
)
from .block_senders import (
    BlockSendersResource,
    AsyncBlockSendersResource,
    BlockSendersResourceWithRawResponse,
    AsyncBlockSendersResourceWithRawResponse,
    BlockSendersResourceWithStreamingResponse,
    AsyncBlockSendersResourceWithStreamingResponse,
)
from .allow_policies import (
    AllowPoliciesResource,
    AsyncAllowPoliciesResource,
    AllowPoliciesResourceWithRawResponse,
    AsyncAllowPoliciesResourceWithRawResponse,
    AllowPoliciesResourceWithStreamingResponse,
    AsyncAllowPoliciesResourceWithStreamingResponse,
)
from .trusted_domains import (
    TrustedDomainsResource,
    AsyncTrustedDomainsResource,
    TrustedDomainsResourceWithRawResponse,
    AsyncTrustedDomainsResourceWithRawResponse,
    TrustedDomainsResourceWithStreamingResponse,
    AsyncTrustedDomainsResourceWithStreamingResponse,
)
from .impersonation_registry import (
    ImpersonationRegistryResource,
    AsyncImpersonationRegistryResource,
    ImpersonationRegistryResourceWithRawResponse,
    AsyncImpersonationRegistryResourceWithRawResponse,
    ImpersonationRegistryResourceWithStreamingResponse,
    AsyncImpersonationRegistryResourceWithStreamingResponse,
)

__all__ = [
    "AllowPoliciesResource",
    "AsyncAllowPoliciesResource",
    "AllowPoliciesResourceWithRawResponse",
    "AsyncAllowPoliciesResourceWithRawResponse",
    "AllowPoliciesResourceWithStreamingResponse",
    "AsyncAllowPoliciesResourceWithStreamingResponse",
    "BlockSendersResource",
    "AsyncBlockSendersResource",
    "BlockSendersResourceWithRawResponse",
    "AsyncBlockSendersResourceWithRawResponse",
    "BlockSendersResourceWithStreamingResponse",
    "AsyncBlockSendersResourceWithStreamingResponse",
    "DomainsResource",
    "AsyncDomainsResource",
    "DomainsResourceWithRawResponse",
    "AsyncDomainsResourceWithRawResponse",
    "DomainsResourceWithStreamingResponse",
    "AsyncDomainsResourceWithStreamingResponse",
    "ImpersonationRegistryResource",
    "AsyncImpersonationRegistryResource",
    "ImpersonationRegistryResourceWithRawResponse",
    "AsyncImpersonationRegistryResourceWithRawResponse",
    "ImpersonationRegistryResourceWithStreamingResponse",
    "AsyncImpersonationRegistryResourceWithStreamingResponse",
    "TrustedDomainsResource",
    "AsyncTrustedDomainsResource",
    "TrustedDomainsResourceWithRawResponse",
    "AsyncTrustedDomainsResourceWithRawResponse",
    "TrustedDomainsResourceWithStreamingResponse",
    "AsyncTrustedDomainsResourceWithStreamingResponse",
    "SettingsResource",
    "AsyncSettingsResource",
    "SettingsResourceWithRawResponse",
    "AsyncSettingsResourceWithRawResponse",
    "SettingsResourceWithStreamingResponse",
    "AsyncSettingsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import date

from ..._models import BaseModel

__all__ = ["Whois"]


class Whois(BaseModel):
    created_date: Optional[date] = None

    domain: Optional[str] = None

    nameservers: Optional[List[str]] = None

    registrant: Optional[str] = None

    registrant_country: Optional[str] = None

    registrant_email: Optional[str] = None

    registrant_org: Optional[str] = None

    registrar: Optional[str] = None

    updated_date: Optional[date] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = ["MatchGetParams"]


class MatchGetParams(TypedDict, total=False):
    account_id: Required[str]

    query_id: Required[SequenceNotStr[str]]
    """Query ID or comma-separated list of Query IDs.

    When multiple IDs are provided, matches are deduplicated across queries and each
    match includes a match_details array with per-match query metadata and dismissed
    state.
    """

    domain_search: str
    """Filter matches by domain name (substring match)"""

    include_dismissed: str

    include_domain_id: str

    limit: str

    offset: str

    order: Literal["asc", "desc"]
    """Sort order. Options: 'asc' (ascending) or 'desc' (descending)"""

    order_by: Annotated[Literal["domain", "first_seen", "registrar"], PropertyInfo(alias="orderBy")]
    """Column to sort by. Options: 'domain', 'first_seen', or 'registrar'"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .purge import (
    PurgeResource,
    AsyncPurgeResource,
    PurgeResourceWithRawResponse,
    AsyncPurgeResourceWithRawResponse,
    PurgeResourceWithStreamingResponse,
    AsyncPurgeResourceWithStreamingResponse,
)
from .queues import (
    QueuesResource,
    AsyncQueuesResource,
    QueuesResourceWithRawResponse,
    AsyncQueuesResourceWithRawResponse,
    QueuesResourceWithStreamingResponse,
    AsyncQueuesResourceWithStreamingResponse,
)
from .messages import (
    MessagesResource,
    AsyncMessagesResource,
    MessagesResourceWithRawResponse,
    AsyncMessagesResourceWithRawResponse,
    MessagesResourceWithStreamingResponse,
    AsyncMessagesResourceWithStreamingResponse,
)
from .consumers import (
    ConsumersResource,
    AsyncConsumersResource,
    ConsumersResourceWithRawResponse,
    AsyncConsumersResourceWithRawResponse,
    ConsumersResourceWithStreamingResponse,
    AsyncConsumersResourceWithStreamingResponse,
)
from .subscriptions import (
    SubscriptionsResource,
    AsyncSubscriptionsResource,
    SubscriptionsResourceWithRawResponse,
    AsyncSubscriptionsResourceWithRawResponse,
    SubscriptionsResourceWithStreamingResponse,
    AsyncSubscriptionsResourceWithStreamingResponse,
)

__all__ = [
    "MessagesResource",
    "AsyncMessagesResource",
    "MessagesResourceWithRawResponse",
    "AsyncMessagesResourceWithRawResponse",
    "MessagesResourceWithStreamingResponse",
    "AsyncMessagesResourceWithStreamingResponse",
    "PurgeResource",
    "AsyncPurgeResource",
    "PurgeResourceWithRawResponse",
    "AsyncPurgeResourceWithRawResponse",
    "PurgeResourceWithStreamingResponse",
    "AsyncPurgeResourceWithStreamingResponse",
    "ConsumersResource",
    "AsyncConsumersResource",
    "ConsumersResourceWithRawResponse",
    "AsyncConsumersResourceWithRawResponse",
    "ConsumersResourceWithStreamingResponse",
    "AsyncConsumersResourceWithStreamingResponse",
    "SubscriptionsResource",
    "AsyncSubscriptionsResource",
    "SubscriptionsResourceWithRawResponse",
    "AsyncSubscriptionsResourceWithRawResponse",
    "SubscriptionsResourceWithStreamingResponse",
    "AsyncSubscriptionsResourceWithStreamingResponse",
    "QueuesResource",
    "AsyncQueuesResource",
    "QueuesResourceWithRawResponse",
    "AsyncQueuesResourceWithRawResponse",
    "QueuesResourceWithStreamingResponse",
    "AsyncQueuesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .action import Action as Action
from .address import Address as Address
from .matcher import Matcher as Matcher
from .settings import Settings as Settings
from .dns_record import DNSRecord as DNSRecord
from .action_param import ActionParam as ActionParam
from .matcher_param import MatcherParam as MatcherParam
from .dns_get_params import DNSGetParams as DNSGetParams
from .dns_edit_params import DNSEditParams as DNSEditParams
from .dns_get_response import DNSGetResponse as DNSGetResponse
from .rule_list_params import RuleListParams as RuleListParams
from .dns_create_params import DNSCreateParams as DNSCreateParams
from .email_routing_rule import EmailRoutingRule as EmailRoutingRule
from .rule_create_params import RuleCreateParams as RuleCreateParams
from .rule_update_params import RuleUpdateParams as RuleUpdateParams
from .address_list_params import AddressListParams as AddressListParams
from .address_create_params import AddressCreateParams as AddressCreateParams
from .email_routing_enable_params import EmailRoutingEnableParams as EmailRoutingEnableParams
from .email_routing_disable_params import EmailRoutingDisableParams as EmailRoutingDisableParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["Traceroute", "Colo", "ColoColo", "ColoHop", "ColoHopNode"]


class ColoColo(BaseModel):
    city: Optional[str] = None
    """Source colo city."""

    name: Optional[str] = None
    """Source colo name."""


class ColoHopNode(BaseModel):
    asn: Optional[str] = None
    """AS number associated with the node object."""

    ip: Optional[str] = None
    """IP address of the node."""

    labels: Optional[List[str]] = None
    """
    Field appears if there is an additional annotation printed when the probe
    returns. Field also appears when running a GRE+ICMP traceroute to denote which
    traceroute a node comes from.
    """

    max_rtt_ms: Optional[float] = None
    """Maximum RTT in ms."""

    mean_rtt_ms: Optional[float] = None
    """Mean RTT in ms."""

    min_rtt_ms: Optional[float] = None
    """Minimum RTT in ms."""

    name: Optional[str] = None
    """Host name of the address, this may be the same as the IP address."""

    packet_count: Optional[int] = None
    """Number of packets with a response from this node."""

    std_dev_rtt_ms: Optional[float] = None
    """Standard deviation of the RTTs in ms."""


class ColoHop(BaseModel):
    nodes: Optional[List[ColoHopNode]] = None
    """An array of node objects."""

    packets_lost: Optional[int] = None
    """Number of packets where no response was received."""

    packets_sent: Optional[int] = None
    """Number of packets sent with specified TTL."""

    packets_ttl: Optional[int] = None
    """The time to live (TTL)."""


class Colo(BaseModel):
    colo: Optional[ColoColo] = None

    error: Optional[
        Literal[
            "",
            "Could not gather traceroute data: Code 1",
            "Could not gather traceroute data: Code 2",
            "Could not gather traceroute data: Code 3",
            "Could not gather traceroute data: Code 4",
        ]
    ] = None
    """Errors resulting from collecting traceroute from colo to target."""

    hops: Optional[List[ColoHop]] = None

    target_summary: Optional[object] = None
    """Aggregated statistics from all hops about the target."""

    traceroute_time_ms: Optional[int] = None
    """Total time of traceroute in ms."""


class Traceroute(BaseModel):
    colos: Optional[List[Colo]] = None

    target: Optional[str] = None
    """The target hostname, IPv6, or IPv6 address."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["OriginErrorPagePassThru"]


class OriginErrorPagePassThru(BaseModel):
    id: Optional[Literal["origin_error_page_pass_thru"]] = None
    """
    Turn on or off Cloudflare error pages generated from issues sent from the origin
    server. If enabled, this setting triggers error pages issued by the origin.
    """

    value: Optional[Literal["on", "off"]] = None
    """The status of Origin Error Page Passthru."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = ["DatabaseImportParams", "Init", "Ingest", "Poll"]


class Init(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier tag."""

    action: Required[Literal["init"]]
    """Indicates you have a new SQL file to upload."""

    etag: Required[str]
    """Required when action is 'init' or 'ingest'.

    An md5 hash of the file you're uploading. Used to check if it already exists,
    and validate its contents before ingesting.
    """


class Ingest(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier tag."""

    action: Required[Literal["ingest"]]
    """Indicates you've finished uploading to tell the D1 to start consuming it"""

    etag: Required[str]
    """An md5 hash of the file you're uploading.

    Used to check if it already exists, and validate its contents before ingesting.
    """

    filename: Required[str]
    """The filename you have successfully uploaded."""


class Poll(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier tag."""

    action: Required[Literal["poll"]]
    """Indicates you've finished uploading to tell the D1 to start consuming it"""

    current_bookmark: Required[str]
    """This identifies the currently-running import, checking its status."""


DatabaseImportParams: TypeAlias = Union[Init, Ingest, Poll]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from ...._models import BaseModel
from .profiles.pattern import Pattern

__all__ = [
    "EntryUpdateResponse",
    "CustomEntry",
    "CustomPromptTopicEntry",
    "PredefinedEntry",
    "PredefinedEntryConfidence",
    "PredefinedEntryVariant",
    "PredefinedEntryVariantUnionMember0",
    "PredefinedEntryVariantUnionMember1",
    "IntegrationEntry",
    "ExactDataEntry",
    "DocumentFingerprintEntry",
    "WordListEntry",
]


class CustomEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    pattern: Pattern

    type: Literal["custom"]

    updated_at: datetime

    description: Optional[str] = None

    profile_id: Optional[str] = None


class CustomPromptTopicEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["custom_prompt_topic"]

    updated_at: datetime


class PredefinedEntryConfidence(BaseModel):
    ai_context_available: bool
    """Indicates whether this entry has AI remote service validation."""

    available: bool
    """
    Indicates whether this entry has any form of validation that is not an AI remote
    service.
    """


class PredefinedEntryVariantUnionMember0(BaseModel):
    """A Predefined AI prompt classification topic entry."""

    topic_type: Literal["Intent", "Content"]

    type: Literal["PromptTopic"]

    description: Optional[str] = None
    """
    A customer-facing explanation of what this predefined AI prompt topic
    represents.
    """


class PredefinedEntryVariantUnionMember1(BaseModel):
    """A general predefined entry."""

    type: Literal["General"]

    description: Optional[str] = None
    """A customer-facing explanation of what this predefined entry represents."""


PredefinedEntryVariant: TypeAlias = Union[PredefinedEntryVariantUnionMember0, PredefinedEntryVariantUnionMember1]


class PredefinedEntry(BaseModel):
    id: str

    confidence: PredefinedEntryConfidence

    enabled: bool

    name: str

    type: Literal["predefined"]

    profile_id: Optional[str] = None

    variant: Optional[PredefinedEntryVariant] = None
    """A Predefined AI prompt classification topic entry."""


class IntegrationEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["integration"]

    updated_at: datetime

    profile_id: Optional[str] = None


class ExactDataEntry(BaseModel):
    id: str

    case_sensitive: bool
    """
    Only applies to custom word lists. Determines if the words should be matched in
    a case-sensitive manner Cannot be set to false if secret is true
    """

    created_at: datetime

    enabled: bool

    name: str

    secret: bool

    type: Literal["exact_data"]

    updated_at: datetime


class DocumentFingerprintEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["document_fingerprint"]

    updated_at: datetime


class WordListEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["word_list"]

    updated_at: datetime

    word_list: object

    profile_id: Optional[str] = None


EntryUpdateResponse: TypeAlias = Union[
    CustomEntry,
    CustomPromptTopicEntry,
    PredefinedEntry,
    IntegrationEntry,
    ExactDataEntry,
    DocumentFingerprintEntry,
    WordListEntry,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .iqi import (
    IQIResource,
    AsyncIQIResource,
    IQIResourceWithRawResponse,
    AsyncIQIResourceWithRawResponse,
    IQIResourceWithStreamingResponse,
    AsyncIQIResourceWithStreamingResponse,
)
from .speed import (
    SpeedResource,
    AsyncSpeedResource,
    SpeedResourceWithRawResponse,
    AsyncSpeedResourceWithRawResponse,
    SpeedResourceWithStreamingResponse,
    AsyncSpeedResourceWithStreamingResponse,
)
from .quality import (
    QualityResource,
    AsyncQualityResource,
    QualityResourceWithRawResponse,
    AsyncQualityResourceWithRawResponse,
    QualityResourceWithStreamingResponse,
    AsyncQualityResourceWithStreamingResponse,
)

__all__ = [
    "IQIResource",
    "AsyncIQIResource",
    "IQIResourceWithRawResponse",
    "AsyncIQIResourceWithRawResponse",
    "IQIResourceWithStreamingResponse",
    "AsyncIQIResourceWithStreamingResponse",
    "SpeedResource",
    "AsyncSpeedResource",
    "SpeedResourceWithRawResponse",
    "AsyncSpeedResourceWithRawResponse",
    "SpeedResourceWithStreamingResponse",
    "AsyncSpeedResourceWithStreamingResponse",
    "QualityResource",
    "AsyncQualityResource",
    "QualityResourceWithRawResponse",
    "AsyncQualityResourceWithRawResponse",
    "QualityResourceWithStreamingResponse",
    "AsyncQualityResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict
from typing_extensions import Literal, TypeAlias

from ...._models import BaseModel

__all__ = ["OperationBulkEditResponse", "OperationBulkEditResponseItem"]


class OperationBulkEditResponseItem(BaseModel):
    mitigation_action: Literal["log", "block", "none"]
    """
    When set, this applies a mitigation action to this operation which supersedes a
    global schema validation setting just for this operation

    - `"log"` - log request when request does not conform to schema for this
      operation
    - `"block"` - deny access to the site when request does not conform to schema
      for this operation
    - `"none"` - will skip mitigation for this operation
    """

    operation_id: str
    """UUID."""


OperationBulkEditResponse: TypeAlias = Dict[str, OperationBulkEditResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .security_txt_get_response import SecurityTXTGetResponse as SecurityTXTGetResponse
from .security_txt_update_params import SecurityTXTUpdateParams as SecurityTXTUpdateParams
from .security_txt_delete_response import SecurityTXTDeleteResponse as SecurityTXTDeleteResponse
from .security_txt_update_response import SecurityTXTUpdateResponse as SecurityTXTUpdateResponse
